@@ -1,5 +1,6 @@
 /* BFD support for the HP Precision Architecture architecture.
-   Copyright 1992, 1995, 1998, 1999, 2000 Free Software Foundation, Inc.
+   Copyright 1992, 1995, 1998, 1999, 2000, 2002, 2003, 2007
+   Free Software Foundation, Inc.
 
 This file is part of BFD, the Binary File Descriptor library.
 
@@ -15,10 +16,10 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
 
-#include "bfd.h"
 #include "sysdep.h"
+#include "bfd.h"
 #include "libbfd.h"
 
 static const bfd_arch_info_type bfd_hppa10_arch =
@@ -27,11 +28,11 @@ static const bfd_arch_info_type bfd_hppa10_arch =
   32,				/* 32 bits in an address */
   8,				/* 8 bits in a byte */
   bfd_arch_hppa,
-  10,				/* By convention PA1.0 = 10 */
+  bfd_mach_hppa10,		/* By convention PA1.0 = 10 */
   "hppa",
   "hppa1.0",
   3,
-  true,				/* Unless we use 1.1 specific features */
+  TRUE,				/* Unless we use 1.1 specific features */
   bfd_default_compatible,
   bfd_default_scan ,
   0,
@@ -44,11 +45,11 @@ static const bfd_arch_info_type bfd_hppa20_arch =
   32,				/* 32 bits in an address */
   8,				/* 8 bits in a byte */
   bfd_arch_hppa,
-  20,				/* By convention PA1.0 = 10 */
+  bfd_mach_hppa20,		/* By convention PA2.0 = 20 */
   "hppa",
   "hppa2.0",
   3,
-  false,			/* Unless we use 1.1 specific features */
+  FALSE,			/* Unless we use 1.1 specific features */
   bfd_default_compatible,
   bfd_default_scan ,
   &bfd_hppa10_arch,
@@ -57,15 +58,15 @@ static const bfd_arch_info_type bfd_hppa20_arch =
 /* PA2.0 in wide mode */
 static const bfd_arch_info_type bfd_hppa20w_arch =
 {
-  64,				/* 32 bits in a word */
-  64,				/* 32 bits in an address */
+  64,				/* 64 bits in a word */
+  64,				/* 64 bits in an address */
   8,				/* 8 bits in a byte */
   bfd_arch_hppa,
-  25,				/* ??? How best to describe wide mode here?  */
+  bfd_mach_hppa20w,		/* ??? How best to describe wide mode here?  */
   "hppa",
   "hppa2.0w",
   3,
-  false,			/* Unless we use 1.1 specific features */
+  FALSE,			/* Unless we use 1.1 specific features */
   bfd_default_compatible,
   bfd_default_scan ,
   &bfd_hppa20_arch,
@@ -77,11 +78,11 @@ const bfd_arch_info_type bfd_hppa_arch =
   32,				/* 32 bits in an address */
   8,				/* 8 bits in a byte */
   bfd_arch_hppa,
-  11,				/* By convention PA1.1 = 11 */
+  bfd_mach_hppa11,		/* By convention PA1.1 = 11 */
   "hppa",
   "hppa1.1",
   3,
-  false,			/* 1.1 specific features used */
+  FALSE,			/* 1.1 specific features used */
   bfd_default_compatible,
   bfd_default_scan ,
   &bfd_hppa20w_arch,
