@@ -1,5 +1,5 @@
 /* BFD support for the MAXQ20/10 architecture.
-   Copyright  2004, 2005  Free Software Foundation, Inc.
+   Copyright 2004, 2005, 2007 Free Software Foundation, Inc.
 
    Written by Vineet Sharma(vineets@noida.hcltech.com)
 	      Inderpreet Singh(inderpreetb@noida.hcltech.com)		
@@ -18,10 +18,10 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
 
-#include "bfd.h"
 #include "sysdep.h"
+#include "bfd.h"
 #include "libbfd.h"
 
 /* MAXQ Archtecture info.  */
