@@ -1,5 +1,5 @@
 /* BFD back-end for PDP-11 support.
-   Copyright 2001 Free Software Foundation, Inc.
+   Copyright 2001, 2002, 2007 Free Software Foundation, Inc.
 
 This file is part of BFD, the Binary File Descriptor library.
 
@@ -15,10 +15,10 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
 
-#include "bfd.h"
 #include "sysdep.h"
+#include "bfd.h"
 #include "libbfd.h"
 
 const bfd_arch_info_type bfd_pdp11_arch =
@@ -31,7 +31,7 @@ const bfd_arch_info_type bfd_pdp11_arch =
     "pdp11",
     "pdp11",
     1,		/* aligment = 16 bit */
-    true, /* the one and only */
+    TRUE, /* the one and only */
     bfd_default_compatible,
     bfd_default_scan ,
     0,
