@@ -1,5 +1,6 @@
 /* bfd back-end for vax support
-   Copyright 1990, 1991, 1994, 2000, 2002 Free Software Foundation, Inc.
+   Copyright 1990, 1991, 1994, 2000, 2002, 2007
+   Free Software Foundation, Inc.
    Written by Steve Chamberlain of Cygnus Support.
 
 This file is part of BFD, the Binary File Descriptor library.
@@ -16,10 +17,10 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
 
-#include "bfd.h"
 #include "sysdep.h"
+#include "bfd.h"
 #include "libbfd.h"
 
 const bfd_arch_info_type bfd_vax_arch =
@@ -34,6 +35,6 @@ const bfd_arch_info_type bfd_vax_arch =
     3,
     TRUE, /* the one and only */
     bfd_default_compatible,
-    bfd_default_scan ,
+    bfd_default_scan,
     0,
   };
