@@ -1,5 +1,6 @@
 /* SuperH SH64-specific support for 32-bit ELF
-   Copyright 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright 2000, 2001, 2002, 2003, 2004, 2007
+   Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -15,12 +16,12 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #define SH64_ELF
 
-#include "bfd.h"
 #include "sysdep.h"
+#include "bfd.h"
 #include "libbfd.h"
 #include "elf-bfd.h"
 #include "elf/sh.h"
@@ -28,15 +29,13 @@
 #include "../opcodes/sh64-opc.h"
 
 static bfd_boolean sh64_address_in_cranges
-  PARAMS ((asection *cranges, bfd_vma, sh64_elf_crange *));
+  (asection *cranges, bfd_vma, sh64_elf_crange *);
 
 /* Ordering functions of a crange, for the qsort and bsearch calls and for
    different endianness.  */
 
 int
-_bfd_sh64_crange_qsort_cmpb (p1, p2)
-     const PTR p1;
-     const PTR p2;
+_bfd_sh64_crange_qsort_cmpb (const void *p1, const void *p2)
 {
   bfd_vma a1 = bfd_getb32 (p1);
   bfd_vma a2 = bfd_getb32 (p2);
@@ -49,9 +48,7 @@ _bfd_sh64_crange_qsort_cmpb (p1, p2)
 }
 
 int
-_bfd_sh64_crange_qsort_cmpl (p1, p2)
-     const PTR p1;
-     const PTR p2;
+_bfd_sh64_crange_qsort_cmpl (const void *p1, const void *p2)
 {
   bfd_vma a1 = (bfd_vma) bfd_getl32 (p1);
   bfd_vma a2 = (bfd_vma) bfd_getl32 (p2);
@@ -64,9 +61,7 @@ _bfd_sh64_crange_qsort_cmpl (p1, p2)
 }
 
 int
-_bfd_sh64_crange_bsearch_cmpb (p1, p2)
-     const PTR p1;
-     const PTR p2;
+_bfd_sh64_crange_bsearch_cmpb (const void *p1, const void *p2)
 {
   bfd_vma a1 = *(bfd_vma *) p1;
   bfd_vma a2 = (bfd_vma) bfd_getb32 (p2);
@@ -81,9 +76,7 @@ _bfd_sh64_crange_bsearch_cmpb (p1, p2)
 }
 
 int
-_bfd_sh64_crange_bsearch_cmpl (p1, p2)
-     const PTR p1;
-     const PTR p2;
+_bfd_sh64_crange_bsearch_cmpl (const void *p1, const void *p2)
 {
   bfd_vma a1 = *(bfd_vma *) p1;
   bfd_vma a2 = (bfd_vma) bfd_getl32 (p2);
@@ -102,14 +95,12 @@ _bfd_sh64_crange_bsearch_cmpl (p1, p2)
    filled into RANGEP if non-NULL.  */
 
 static bfd_boolean
-sh64_address_in_cranges (cranges, addr, rangep)
-     asection *cranges;
-     bfd_vma addr;
-     sh64_elf_crange *rangep;
+sh64_address_in_cranges (asection *cranges, bfd_vma addr,
+			 sh64_elf_crange *rangep)
 {
   bfd_byte *cranges_contents;
   bfd_byte *found_rangep;
-  bfd_size_type cranges_size = bfd_section_size (cranges->owner, cranges);
+  bfd_size_type cranges_size = cranges->size;
 
   /* If the size is not a multiple of the cranges entry size, then
      something is badly wrong.  */
@@ -127,15 +118,8 @@ sh64_address_in_cranges (cranges, addr, rangep)
     cranges_contents = cranges->contents;
   else
     {
-      cranges_contents
-	= bfd_malloc (cranges->_cooked_size != 0
-		      ? cranges->_cooked_size : cranges->_raw_size);
-      if (cranges_contents == NULL)
-	return FALSE;
-
-      if (! bfd_get_section_contents (cranges->owner, cranges,
-				      cranges_contents, (file_ptr) 0,
-				      cranges_size))
+      if (!bfd_malloc_and_get_section (cranges->owner, cranges,
+				       &cranges_contents))
 	goto error_return;
 
       /* Is it sorted?  */
@@ -192,7 +176,8 @@ sh64_address_in_cranges (cranges, addr, rangep)
   return FALSE;
 
 error_return:
-  free (cranges_contents);
+  if (cranges_contents != NULL)
+    free (cranges_contents);
   return FALSE;
 }
 
@@ -200,10 +185,7 @@ error_return:
    *RANGEP if it's non-NULL.  */
 
 enum sh64_elf_cr_type
-sh64_get_contents_type (sec, addr, rangep)
-     asection *sec;
-     bfd_vma addr;
-     sh64_elf_crange *rangep;
+sh64_get_contents_type (asection *sec, bfd_vma addr, sh64_elf_crange *rangep)
 {
   asection *cranges;
 
@@ -212,7 +194,7 @@ sh64_get_contents_type (sec, addr, rangep)
       && elf_elfheader (sec->owner)->e_type == ET_EXEC)
     {
       rangep->cr_addr = bfd_get_section_vma (sec->owner, sec);
-      rangep->cr_size = bfd_section_size (sec->owner, sec);
+      rangep->cr_size = sec->size;
       rangep->cr_type = CRT_NONE;
     }
   else
@@ -256,9 +238,7 @@ sh64_get_contents_type (sec, addr, rangep)
 /* This is a simpler exported interface for the benefit of gdb et al.  */
 
 bfd_boolean
-sh64_address_is_shmedia (sec, addr)
-     asection *sec;
-     bfd_vma addr;
+sh64_address_is_shmedia (asection *sec, bfd_vma addr)
 {
   sh64_elf_crange dummy;
   return sh64_get_contents_type (sec, addr, &dummy) == CRT_SH5_ISA32;
