@@ -1,55 +1,52 @@
 /* BFD COFF object file private structure.
    Copyright 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
-   2000, 2001, 2002
+   2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
    Written by Cygnus Support.
 
-This file is part of BFD, the Binary File Descriptor library.
+   This file is part of BFD, the Binary File Descriptor library.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include "bfdlink.h"
 
-/* Object file tdata; access macros */
+/* Object file tdata; access macros.  */
 
-#define coff_data(bfd)		((bfd)->tdata.coff_obj_data)
-#define exec_hdr(bfd)		(coff_data(bfd)->hdr)
-#define obj_pe(bfd)             (coff_data(bfd)->pe)
-#define obj_symbols(bfd)	(coff_data(bfd)->symbols)
-#define	obj_sym_filepos(bfd)	(coff_data(bfd)->sym_filepos)
-
-#define obj_relocbase(bfd)	(coff_data(bfd)->relocbase)
-#define obj_raw_syments(bfd)	(coff_data(bfd)->raw_syments)
-#define obj_raw_syment_count(bfd)	(coff_data(bfd)->raw_syment_count)
-#define obj_convert(bfd)	(coff_data(bfd)->conversion_table)
-#define obj_conv_table_size(bfd) (coff_data(bfd)->conv_table_size)
-
-#define obj_coff_external_syms(bfd) (coff_data (bfd)->external_syms)
-#define obj_coff_keep_syms(bfd)	(coff_data (bfd)->keep_syms)
-#define obj_coff_strings(bfd)	(coff_data (bfd)->strings)
-#define obj_coff_keep_strings(bfd) (coff_data (bfd)->keep_strings)
-#define obj_coff_sym_hashes(bfd) (coff_data (bfd)->sym_hashes)
+#define coff_data(bfd)		      ((bfd)->tdata.coff_obj_data)
+#define exec_hdr(bfd)		      (coff_data (bfd)->hdr)
+#define obj_pe(bfd)                   (coff_data (bfd)->pe)
+#define obj_symbols(bfd)	      (coff_data (bfd)->symbols)
+#define	obj_sym_filepos(bfd)	      (coff_data (bfd)->sym_filepos)
+#define obj_relocbase(bfd)	      (coff_data (bfd)->relocbase)
+#define obj_raw_syments(bfd)	      (coff_data (bfd)->raw_syments)
+#define obj_raw_syment_count(bfd)     (coff_data (bfd)->raw_syment_count)
+#define obj_convert(bfd)	      (coff_data (bfd)->conversion_table)
+#define obj_conv_table_size(bfd)      (coff_data (bfd)->conv_table_size)
+#define obj_coff_external_syms(bfd)   (coff_data (bfd)->external_syms)
+#define obj_coff_keep_syms(bfd)	      (coff_data (bfd)->keep_syms)
+#define obj_coff_strings(bfd)	      (coff_data (bfd)->strings)
+#define obj_coff_keep_strings(bfd)    (coff_data (bfd)->keep_strings)
+#define obj_coff_sym_hashes(bfd)      (coff_data (bfd)->sym_hashes)
 #define obj_coff_strings_written(bfd) (coff_data (bfd)->strings_written)
-
-#define obj_coff_local_toc_table(bfd) (coff_data(bfd)->local_toc_sym_map)
+#define obj_coff_local_toc_table(bfd) (coff_data (bfd)->local_toc_sym_map)
 
 /* `Tdata' information kept for COFF files.  */
 
 typedef struct coff_tdata
 {
-  struct   coff_symbol_struct *symbols;	/* symtab for input bfd */
+  struct coff_symbol_struct *symbols;	/* Symtab for input bfd.  */
   unsigned int *conversion_table;
   int conv_table_size;
   file_ptr sym_filepos;
@@ -57,7 +54,7 @@ typedef struct coff_tdata
   struct coff_ptr_struct *raw_syments;
   unsigned long raw_syment_count;
 
-  /* These are only valid once writing has begun */
+  /* These are only valid once writing has begun.  */
   long int relocbase;
 
   /* These members communicate important constants about the symbol table
@@ -73,33 +70,33 @@ typedef struct coff_tdata
 
   /* The unswapped external symbols.  May be NULL.  Read by
      _bfd_coff_get_external_symbols.  */
-  PTR external_syms;
-  /* If this is true, the external_syms may not be freed.  */
-  boolean keep_syms;
+  void * external_syms;
+  /* If this is TRUE, the external_syms may not be freed.  */
+  bfd_boolean keep_syms;
 
   /* The string table.  May be NULL.  Read by
      _bfd_coff_read_string_table.  */
   char *strings;
-  /* If this is true, the strings may not be freed.  */
-  boolean keep_strings;
-  /* If this is true, the strings have been written out already.  */
-  boolean strings_written;
+  /* If this is TRUE, the strings may not be freed.  */
+  bfd_boolean keep_strings;
+  /* If this is TRUE, the strings have been written out already.  */
+  bfd_boolean strings_written;
 
-  /* is this a PE format coff file */
+  /* Is this a PE format coff file?  */
   int pe;
   /* Used by the COFF backend linker.  */
   struct coff_link_hash_entry **sym_hashes;
 
-  /* used by the pe linker for PowerPC */
+  /* Used by the pe linker for PowerPC.  */
   int *local_toc_sym_map;
 
   struct bfd_link_info *link_info;
 
   /* Used by coff_find_nearest_line.  */
-  PTR line_info;
+  void * line_info;
 
-  /* A place to stash dwarf2 info for this bfd. */
-  PTR dwarf2_find_line_info;
+  /* A place to stash dwarf2 info for this bfd.  */
+  void * dwarf2_find_line_info;
 
   /* The timestamp from the COFF file header.  */
   long timestamp;
@@ -110,17 +107,17 @@ typedef struct coff_tdata
 
 } coff_data_type;
 
-/* Tdata for pe image files. */
+/* Tdata for pe image files.  */
 typedef struct pe_tdata
 {
   coff_data_type coff;
   struct internal_extra_pe_aouthdr pe_opthdr;
   int dll;
   int has_reloc_section;
-  boolean (*in_reloc_p) PARAMS((bfd *, reloc_howto_type *));
+  bfd_boolean (*in_reloc_p) (bfd *, reloc_howto_type *);
   flagword real_flags;
   int target_subsystem;
-  boolean force_minimum_alignment;
+  bfd_boolean force_minimum_alignment;
 } pe_data_type;
 
 #define pe_data(bfd)		((bfd)->tdata.pe_obj_data)
@@ -132,11 +129,11 @@ struct xcoff_tdata
   /* Basic COFF information.  */
   coff_data_type coff;
 
-  /* True if this is an XCOFF64 file. */
-  boolean xcoff64;
+  /* TRUE if this is an XCOFF64 file. */
+  bfd_boolean xcoff64;
 
-  /* True if a large a.out header should be generated.  */
-  boolean full_aouthdr;
+  /* TRUE if a large a.out header should be generated.  */
+  bfd_boolean full_aouthdr;
 
   /* TOC value.  */
   bfd_vma toc;
@@ -174,7 +171,7 @@ struct xcoff_tdata
 #define xcoff_data(abfd) ((abfd)->tdata.xcoff_obj_data)
 
 /* We take the address of the first element of an asymbol to ensure that the
- * macro is only ever applied to an asymbol.  */
+   macro is only ever applied to an asymbol.  */
 #define coffsymbol(asymbol) ((coff_symbol_type *)(&((asymbol)->the_bfd)))
 
 /* The used_by_bfd field of a section may be set to a pointer to this
@@ -184,21 +181,23 @@ struct coff_section_tdata
 {
   /* The relocs, swapped into COFF internal form.  This may be NULL.  */
   struct internal_reloc *relocs;
-  /* If this is true, the relocs entry may not be freed.  */
-  boolean keep_relocs;
+  /* If this is TRUE, the relocs entry may not be freed.  */
+  bfd_boolean keep_relocs;
   /* The section contents.  This may be NULL.  */
   bfd_byte *contents;
-  /* If this is true, the contents entry may not be freed.  */
-  boolean keep_contents;
+  /* If this is TRUE, the contents entry may not be freed.  */
+  bfd_boolean keep_contents;
   /* Information cached by coff_find_nearest_line.  */
   bfd_vma offset;
   unsigned int i;
   const char *function;
+  /* Optional information about a COMDAT entry; NULL if not COMDAT. */
+  struct coff_comdat_info *comdat;
   int line_base;
   /* A pointer used for .stab linking optimizations.  */
-  PTR stab_info;
+  void * stab_info;
   /* Available for individual backends.  */
-  PTR tdata;
+  void * tdata;
 };
 
 /* An accessor macro for the coff_section_tdata structure.  */
@@ -276,7 +275,7 @@ struct coff_link_hash_table
 {
   struct bfd_link_hash_table root;
   /* A pointer to information used to link stabs in sections.  */
-  PTR stab_info;
+  struct stab_info stab_info;
 };
 
 /* Look up an entry in a COFF linker hash table.  */
@@ -291,7 +290,7 @@ struct coff_link_hash_table
 #define coff_link_hash_traverse(table, func, info)			\
   (bfd_link_hash_traverse						\
    (&(table)->root,							\
-    (boolean (*) PARAMS ((struct bfd_link_hash_entry *, PTR))) (func),	\
+    (bfd_boolean (*) (struct bfd_link_hash_entry *, void *)) (func), \
     (info)))
 
 /* Get the COFF linker hash table from a link_info structure.  */
@@ -300,67 +299,69 @@ struct coff_link_hash_table
 
 /* Functions in coffgen.c.  */
 extern const bfd_target *coff_object_p
-  PARAMS ((bfd *));
-extern struct sec *coff_section_from_bfd_index
-  PARAMS ((bfd *, int));
+  (bfd *);
+extern struct bfd_section *coff_section_from_bfd_index
+  (bfd *, int);
 extern long coff_get_symtab_upper_bound
-  PARAMS ((bfd *));
-extern long coff_get_symtab
-  PARAMS ((bfd *, asymbol **));
+  (bfd *);
+extern long coff_canonicalize_symtab
+  (bfd *, asymbol **);
 extern int coff_count_linenumbers
-  PARAMS ((bfd *));
+  (bfd *);
 extern struct coff_symbol_struct *coff_symbol_from
-  PARAMS ((bfd *, asymbol *));
-extern boolean coff_renumber_symbols
-  PARAMS ((bfd *, int *));
+  (bfd *, asymbol *);
+extern bfd_boolean coff_renumber_symbols
+  (bfd *, int *);
 extern void coff_mangle_symbols
-  PARAMS ((bfd *));
-extern boolean coff_write_symbols
-  PARAMS ((bfd *));
-extern boolean coff_write_linenumbers
-  PARAMS ((bfd *));
+  (bfd *);
+extern bfd_boolean coff_write_symbols
+  (bfd *);
+extern bfd_boolean coff_write_linenumbers
+  (bfd *);
 extern alent *coff_get_lineno
-  PARAMS ((bfd *, asymbol *));
+  (bfd *, asymbol *);
 extern asymbol *coff_section_symbol
-  PARAMS ((bfd *, char *));
-extern boolean _bfd_coff_get_external_symbols
-  PARAMS ((bfd *));
+  (bfd *, char *);
+extern bfd_boolean _bfd_coff_get_external_symbols
+  (bfd *);
 extern const char *_bfd_coff_read_string_table
-  PARAMS ((bfd *));
-extern boolean _bfd_coff_free_symbols
-  PARAMS ((bfd *));
+  (bfd *);
+extern bfd_boolean _bfd_coff_free_symbols
+  (bfd *);
 extern struct coff_ptr_struct *coff_get_normalized_symtab
-  PARAMS ((bfd *));
+  (bfd *);
 extern long coff_get_reloc_upper_bound
-  PARAMS ((bfd *, sec_ptr));
+  (bfd *, sec_ptr);
 extern asymbol *coff_make_empty_symbol
-  PARAMS ((bfd *));
+  (bfd *);
 extern void coff_print_symbol
-  PARAMS ((bfd *, PTR filep, asymbol *, bfd_print_symbol_type));
+  (bfd *, void * filep, asymbol *, bfd_print_symbol_type);
 extern void coff_get_symbol_info
-  PARAMS ((bfd *, asymbol *, symbol_info *ret));
-extern boolean _bfd_coff_is_local_label_name
-  PARAMS ((bfd *, const char *));
+  (bfd *, asymbol *, symbol_info *ret);
+extern bfd_boolean _bfd_coff_is_local_label_name
+  (bfd *, const char *);
 extern asymbol *coff_bfd_make_debug_symbol
-  PARAMS ((bfd *, PTR, unsigned long));
-extern boolean coff_find_nearest_line
-  PARAMS ((bfd *, asection *, asymbol **, bfd_vma, const char **,
-	   const char **, unsigned int *));
+  (bfd *, void *, unsigned long);
+extern bfd_boolean coff_find_nearest_line
+  (bfd *, asection *, asymbol **, bfd_vma, const char **,
+   const char **, unsigned int *);
+extern bfd_boolean coff_find_inliner_info
+  (bfd *, const char **, const char **, unsigned int *);
 extern int coff_sizeof_headers
-  PARAMS ((bfd *, boolean));
-extern boolean bfd_coff_reloc16_relax_section
-  PARAMS ((bfd *, asection *, struct bfd_link_info *, boolean *));
+  (bfd *, struct bfd_link_info *);
+extern bfd_boolean bfd_coff_reloc16_relax_section
+  (bfd *, asection *, struct bfd_link_info *, bfd_boolean *);
 extern bfd_byte *bfd_coff_reloc16_get_relocated_section_contents
-  PARAMS ((bfd *, struct bfd_link_info *, struct bfd_link_order *,
-	   bfd_byte *, boolean, asymbol **));
+  (bfd *, struct bfd_link_info *, struct bfd_link_order *,
+   bfd_byte *, bfd_boolean, asymbol **);
 extern bfd_vma bfd_coff_reloc16_get_value
-   PARAMS ((arelent *, struct bfd_link_info *, asection *));
+  (arelent *, struct bfd_link_info *, asection *);
 extern void bfd_perform_slip
-  PARAMS ((bfd *, unsigned int, asection *, bfd_vma));
+  (bfd *, unsigned int, asection *, bfd_vma);
 
 /* Functions and types in cofflink.c.  */
 
-#define STRING_SIZE_SIZE (4)
+#define STRING_SIZE_SIZE 4
 
 /* We use a hash table to merge identical enum, struct, and union
    definitions in the linker.  */
@@ -420,7 +421,8 @@ struct coff_debug_merge_hash_table
 /* Initialize a COFF debug merge hash table.  */
 
 #define coff_debug_merge_hash_table_init(table) \
-  (bfd_hash_table_init (&(table)->root, _bfd_coff_debug_merge_hash_newfunc))
+  (bfd_hash_table_init (&(table)->root, _bfd_coff_debug_merge_hash_newfunc, \
+			sizeof (struct coff_debug_merge_hash_entry)))
 
 /* Free a COFF debug merge hash table.  */
 
@@ -434,7 +436,7 @@ struct coff_debug_merge_hash_table
    bfd_hash_lookup (&(table)->root, (string), (create), (copy)))
 
 /* Information we keep for each section in the output file when doing
-   a relocateable link.  */
+   a relocatable link.  */
 
 struct coff_link_section_info
 {
@@ -454,14 +456,14 @@ struct coff_final_link_info
   /* Output BFD.  */
   bfd *output_bfd;
   /* Used to indicate failure in traversal routine.  */
-  boolean failed;
+  bfd_boolean failed;
   /* If doing "task linking" set only during the time when we want the
      global symbol writer to convert the storage class of defined global
      symbols from global to static. */
-  boolean global_to_static;
+  bfd_boolean global_to_static;
   /* Hash table for long symbol names.  */
   struct bfd_strtab_hash *strtab;
-  /* When doing a relocateable link, an array of information kept for
+  /* When doing a relocatable link, an array of information kept for
      each output section, indexed by the target_index field.  */
   struct coff_link_section_info *section_info;
   /* Symbol index of last C_FILE symbol (-1 if none).  */
@@ -531,38 +533,38 @@ struct coff_section_alignment_entry
 };
 
 extern struct bfd_hash_entry *_bfd_coff_link_hash_newfunc
-  PARAMS ((struct bfd_hash_entry *, struct bfd_hash_table *, const char *));
-extern boolean _bfd_coff_link_hash_table_init
-  PARAMS ((struct coff_link_hash_table *, bfd *,
-	   struct bfd_hash_entry *(*) (struct bfd_hash_entry *,
-				       struct bfd_hash_table *,
-				       const char *)));
+  (struct bfd_hash_entry *, struct bfd_hash_table *, const char *);
+extern bfd_boolean _bfd_coff_link_hash_table_init
+  (struct coff_link_hash_table *, bfd *,
+   struct bfd_hash_entry *(*) (struct bfd_hash_entry *,
+			       struct bfd_hash_table *,
+			       const char *),
+   unsigned int);
 extern struct bfd_link_hash_table *_bfd_coff_link_hash_table_create
-  PARAMS ((bfd *));
+  (bfd *);
 extern const char *_bfd_coff_internal_syment_name
-  PARAMS ((bfd *, const struct internal_syment *, char *));
-extern boolean _bfd_coff_link_add_symbols
-  PARAMS ((bfd *, struct bfd_link_info *));
-extern boolean _bfd_coff_final_link
-  PARAMS ((bfd *, struct bfd_link_info *));
+  (bfd *, const struct internal_syment *, char *);
+extern bfd_boolean _bfd_coff_link_add_symbols
+  (bfd *, struct bfd_link_info *);
+extern bfd_boolean _bfd_coff_final_link
+  (bfd *, struct bfd_link_info *);
 extern struct internal_reloc *_bfd_coff_read_internal_relocs
-  PARAMS ((bfd *, asection *, boolean, bfd_byte *, boolean,
-	   struct internal_reloc *));
-extern boolean _bfd_coff_generic_relocate_section
-  PARAMS ((bfd *, struct bfd_link_info *, bfd *, asection *, bfd_byte *,
-	   struct internal_reloc *, struct internal_syment *, asection **));
-
+  (bfd *, asection *, bfd_boolean, bfd_byte *, bfd_boolean,
+   struct internal_reloc *);
+extern bfd_boolean _bfd_coff_generic_relocate_section
+  (bfd *, struct bfd_link_info *, bfd *, asection *, bfd_byte *,
+   struct internal_reloc *, struct internal_syment *, asection **);
 extern struct bfd_hash_entry *_bfd_coff_debug_merge_hash_newfunc
-  PARAMS ((struct bfd_hash_entry *, struct bfd_hash_table *, const char *));
-extern boolean _bfd_coff_write_global_sym
-  PARAMS ((struct coff_link_hash_entry *, PTR));
-extern boolean _bfd_coff_write_task_globals
-  PARAMS ((struct coff_link_hash_entry *, PTR));
-extern boolean _bfd_coff_link_input_bfd
-  PARAMS ((struct coff_final_link_info *, bfd *));
-extern boolean _bfd_coff_reloc_link_order
-  PARAMS ((bfd *, struct coff_final_link_info *, asection *,
-	   struct bfd_link_order *));
+  (struct bfd_hash_entry *, struct bfd_hash_table *, const char *);
+extern bfd_boolean _bfd_coff_write_global_sym
+  (struct coff_link_hash_entry *, void *);
+extern bfd_boolean _bfd_coff_write_task_globals
+  (struct coff_link_hash_entry *, void *);
+extern bfd_boolean _bfd_coff_link_input_bfd
+  (struct coff_final_link_info *, bfd *);
+extern bfd_boolean _bfd_coff_reloc_link_order
+  (bfd *, struct coff_final_link_info *, asection *,
+   struct bfd_link_order *);
 
 
 #define coff_get_section_contents_in_window \
@@ -571,30 +573,30 @@ extern boolean _bfd_coff_reloc_link_order
 /* Functions in xcofflink.c.  */
 
 extern long _bfd_xcoff_get_dynamic_symtab_upper_bound
-  PARAMS ((bfd *));
+  (bfd *);
 extern long _bfd_xcoff_canonicalize_dynamic_symtab
-  PARAMS ((bfd *, asymbol **));
+  (bfd *, asymbol **);
 extern long _bfd_xcoff_get_dynamic_reloc_upper_bound
-  PARAMS ((bfd *));
+  (bfd *);
 extern long _bfd_xcoff_canonicalize_dynamic_reloc
-  PARAMS ((bfd *, arelent **, asymbol **));
+  (bfd *, arelent **, asymbol **);
 extern struct bfd_link_hash_table *_bfd_xcoff_bfd_link_hash_table_create
-  PARAMS ((bfd *));
+  (bfd *);
 extern void _bfd_xcoff_bfd_link_hash_table_free
-  PARAMS ((struct bfd_link_hash_table *));
-extern boolean _bfd_xcoff_bfd_link_add_symbols
-  PARAMS ((bfd *, struct bfd_link_info *));
-extern boolean _bfd_xcoff_bfd_final_link
-  PARAMS ((bfd *, struct bfd_link_info *));
-extern boolean _bfd_ppc_xcoff_relocate_section
-  PARAMS ((bfd *, struct bfd_link_info *, bfd *, asection *, bfd_byte *,
-	   struct internal_reloc *, struct internal_syment *, asection **));
+  (struct bfd_link_hash_table *);
+extern bfd_boolean _bfd_xcoff_bfd_link_add_symbols
+  (bfd *, struct bfd_link_info *);
+extern bfd_boolean _bfd_xcoff_bfd_final_link
+  (bfd *, struct bfd_link_info *);
+extern bfd_boolean _bfd_ppc_xcoff_relocate_section
+  (bfd *, struct bfd_link_info *, bfd *, asection *, bfd_byte *,
+   struct internal_reloc *, struct internal_syment *, asection **);
 
 /* Functions in coff-ppc.c.  FIXME: These are called be pe.em in the
    linker, and so should start with bfd and be declared in bfd.h.  */
 
-extern boolean ppc_allocate_toc_section
-  PARAMS ((struct bfd_link_info *));
-extern boolean ppc_process_before_allocation
-  PARAMS ((bfd *, struct bfd_link_info *));
+extern bfd_boolean ppc_allocate_toc_section
+  (struct bfd_link_info *);
+extern bfd_boolean ppc_process_before_allocation
+  (bfd *, struct bfd_link_info *);
 
