@@ -1,6 +1,6 @@
 /* arsup.c - Archive support for MRI compatibility
-   Copyright 1992, 1994, 1995, 1996, 1997, 2000, 2002
-   Free Software Foundation, Inc.
+   Copyright 1992, 1994, 1995, 1996, 1997, 1999, 2000, 2001, 2002, 2003,
+   2004, 2007 Free Software Foundation, Inc.
 
    This file is part of GNU Binutils.
 
@@ -16,7 +16,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 
 /* Contributed by Steve Chamberlain
@@ -25,24 +25,26 @@
    This file looks after requests from arparse.y, to provide the MRI
    style librarian command syntax + 1 word LIST.  */
 
+#include "sysdep.h"
 #include "bfd.h"
-#include "arsup.h"
 #include "libiberty.h"
-#include "bucomm.h"
 #include "filenames.h"
+#include "bucomm.h"
+#include "arsup.h"
 
 static void map_over_list
-  PARAMS ((bfd *, void (*function) (bfd *, bfd *), struct list *));
-static void ar_directory_doer PARAMS ((bfd *, bfd *));
-static void ar_addlib_doer PARAMS ((bfd *, bfd *));
+  (bfd *, void (*function) (bfd *, bfd *), struct list *);
+static void ar_directory_doer (bfd *, bfd *);
+static void ar_addlib_doer (bfd *, bfd *);
 
 extern int verbose;
 
+static bfd *obfd;
+static char *real_name;
+static FILE *outfile;
+
 static void
-map_over_list (arch, function, list)
-     bfd *arch;
-     void (*function) PARAMS ((bfd *, bfd *));
-     struct list *list;
+map_over_list (bfd *arch, void (*function) (bfd *, bfd *), struct list *list)
 {
   bfd *head;
 
@@ -69,7 +71,7 @@ map_over_list (arch, function, list)
 	 want to hack multiple references.  */
       for (ptr = list; ptr; ptr = ptr->next)
 	{
-	  boolean found = false;
+	  bfd_boolean found = FALSE;
 	  bfd *prev = arch;
 
 	  for (head = arch->next; head; head = head->next)
@@ -77,7 +79,7 @@ map_over_list (arch, function, list)
 	      if (head->filename != NULL
 		  && FILENAME_CMP (ptr->name, head->filename) == 0)
 		{
-		  found = true;
+		  found = TRUE;
 		  function (head, prev);
 		}
 	      prev = head;
@@ -89,21 +91,15 @@ map_over_list (arch, function, list)
 }
 
 
-FILE *outfile;
 
 static void
-ar_directory_doer (abfd, ignore)
-     bfd *abfd;
-     bfd *ignore ATTRIBUTE_UNUSED;
+ar_directory_doer (bfd *abfd, bfd *ignore ATTRIBUTE_UNUSED)
 {
   print_arelt_descr(outfile, abfd, verbose);
 }
 
 void
-ar_directory (ar_name, list, output)
-     char *ar_name;
-     struct list *list;
-     char *output;
+ar_directory (char *ar_name, struct list *list, char *output)
 {
   bfd *arch;
 
@@ -130,7 +126,7 @@ ar_directory (ar_name, list, output)
 }
 
 void
-DEFUN_VOID(prompt)
+prompt (void)
 {
   extern int interactive;
 
@@ -142,20 +138,15 @@ DEFUN_VOID(prompt)
 }
 
 void
-maybequit ()
+maybequit (void)
 {
   if (! interactive)
     xexit (9);
 }
 
 
-bfd *obfd;
-char *real_name;
-
 void
-ar_open (name, t)
-  char *name;
-  int t;
+ar_open (char *name, int t)
 {
   char *tname = (char *) xmalloc (strlen (name) + 10);
   const char *bname = lbasename (name);
@@ -192,7 +183,7 @@ ar_open (name, t)
 	      return;
 	    }
 
-	  if (bfd_check_format(ibfd, bfd_archive) != true)
+	  if (!bfd_check_format(ibfd, bfd_archive))
 	    {
 	      fprintf (stderr,
 		       _("%s: file %s is not an archive\n"),
@@ -219,9 +210,7 @@ ar_open (name, t)
 }
 
 static void
-ar_addlib_doer (abfd, prev)
-     bfd *abfd;
-     bfd *prev;
+ar_addlib_doer (bfd *abfd, bfd *prev)
 {
   /* Add this module to the output bfd.  */
   if (prev != NULL)
@@ -232,9 +221,7 @@ ar_addlib_doer (abfd, prev)
 }
 
 void
-ar_addlib (name, list)
-     char *name;
-     struct list *list;
+ar_addlib (char *name, struct list *list)
 {
   if (obfd == NULL)
     {
@@ -249,13 +236,12 @@ ar_addlib (name, list)
       if (arch != NULL)
 	map_over_list (arch, ar_addlib_doer, list);
 
-      /* Don't close the bfd, since it will make the elements disasppear.  */
+      /* Don't close the bfd, since it will make the elements disappear.  */
     }
 }
 
 void
-ar_addmod (list)
-     struct list *list;
+ar_addmod (struct list *list)
 {
   if (!obfd)
     {
@@ -286,15 +272,14 @@ ar_addmod (list)
 
 
 void
-ar_clear ()
+ar_clear (void)
 {
   if (obfd)
     obfd->archive_head = 0;
 }
 
 void
-ar_delete (list)
-     struct list *list;
+ar_delete (struct list *list)
 {
   if (!obfd)
     {
@@ -336,7 +321,7 @@ ar_delete (list)
 }
 
 void
-ar_save ()
+ar_save (void)
 {
   if (!obfd)
     {
@@ -349,15 +334,14 @@ ar_save ()
 
       bfd_close (obfd);
 
-      rename (ofilename, real_name);
+      smart_rename (ofilename, real_name, 0);
       obfd = 0;
       free (ofilename);
     }
 }
 
 void
-ar_replace (list)
-     struct list *list;
+ar_replace (struct list *list)
 {
   if (!obfd)
     {
@@ -423,7 +407,7 @@ ar_replace (list)
 
 /* And I added this one.  */
 void
-ar_list ()
+ar_list (void)
 {
   if (!obfd)
     {
@@ -446,18 +430,17 @@ ar_list ()
 }
 
 void
-ar_end ()
+ar_end (void)
 {
   if (obfd)
     {
-      fclose ((FILE *)(obfd->iostream));
+      bfd_cache_close (obfd);
       unlink (bfd_get_filename (obfd));
     }
 }
 
 void
-ar_extract (list)
-     struct list *list;
+ar_extract (struct list *list)
 {
   if (!obfd)
     {
