@@ -1,6 +1,6 @@
 /* objdump.c -- dump information about an object file.
    Copyright 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
-   2000, 2001, 2002, 2003, 2004, 2005, 2006
+   2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
    Free Software Foundation, Inc.
 
    This file is part of GNU Binutils.
@@ -48,12 +48,11 @@
    disassembling is done by the libopcodes library, via a function pointer
    supplied by the disassembler() function.  */
 
+#include "sysdep.h"
 #include "bfd.h"
-#include "bfdver.h"
 #include "progress.h"
 #include "bucomm.h"
 #include "dwarf.h"
-#include "budemang.h"
 #include "getopt.h"
 #include "safe-ctype.h"
 #include "dis-asm.h"
@@ -207,9 +206,10 @@ usage (FILE *stream, int status)
       --file-start-context       Include context from start of file (with -S)\n\
   -I, --include=DIR              Add DIR to search list for source files\n\
   -l, --line-numbers             Include line numbers and filenames in output\n\
-  -C, --demangle[=%s]\n\
-                                 Decode mangled/processed symbol names\n\
-      --demangler=<dso:function> Set dso and demangler function\n\
+  -C, --demangle[=STYLE]         Decode mangled/processed symbol names\n\
+                                  The STYLE, if specified, can be `auto', `gnu',\n\
+                                  `lucid', `arm', `hp', `edg', `gnu-v3', `java'\n\
+                                  or `gnat'\n\
   -w, --wide                     Format output for more than 80 columns\n\
   -z, --disassemble-zeroes       Do not skip blocks of zeroes when disassembling\n\
       --start-address=ADDR       Only process data whose address is >= ADDR\n\
@@ -218,13 +218,13 @@ usage (FILE *stream, int status)
       --[no-]show-raw-insn       Display hex alongside symbolic disassembly\n\
       --adjust-vma=OFFSET        Add OFFSET to all displayed section addresses\n\
       --special-syms             Include special symbols in symbol dumps\n\
-\n"), get_demangler_list ());
+\n"));
       list_supported_targets (program_name, stream);
       list_supported_architectures (program_name, stream);
 
       disassembler_usage (stream);
     }
-  if (status == 0)
+  if (REPORT_BUGS_TO[0] && status == 0)
     fprintf (stream, _("Report bugs to %s.\n"), REPORT_BUGS_TO);
   exit (status);
 }
@@ -235,8 +235,7 @@ enum option_values
     OPTION_ENDIAN=150,
     OPTION_START_ADDRESS,
     OPTION_STOP_ADDRESS,
-    OPTION_ADJUST_VMA,
-    OPTION_DEMANGLER
+    OPTION_ADJUST_VMA
   };
 
 static struct option long_options[]=
@@ -249,7 +248,6 @@ static struct option long_options[]=
   {"debugging", no_argument, NULL, 'g'},
   {"debugging-tags", no_argument, NULL, 'e'},
   {"demangle", optional_argument, NULL, 'C'},
-  {"demangler", required_argument, NULL, OPTION_DEMANGLER},
   {"disassemble", no_argument, NULL, 'd'},
   {"disassemble-all", no_argument, NULL, 'D'},
   {"disassembler-options", required_argument, NULL, 'M'},
@@ -651,8 +649,9 @@ objdump_print_symname (bfd *abfd, struct disassemble_info *info,
   if (do_demangle && name[0] != '\0')
     {
       /* Demangle the name.  */
-      alloc = demangle (abfd, name);
-      name = alloc;
+      alloc = bfd_demangle (abfd, name, DMGL_ANSI | DMGL_PARAMS);
+      if (alloc != NULL)
+	name = alloc;
     }
 
   if (info != NULL)
@@ -688,6 +687,7 @@ find_symbol_for_address (bfd_vma vma,
   bfd *abfd;
   asection *sec;
   unsigned int opb;
+  bfd_boolean want_section;
 
   if (sorted_symcount < 1)
     return NULL;
@@ -733,15 +733,19 @@ find_symbol_for_address (bfd_vma vma,
      Note that this may be wrong for some symbol references if the
      sections have overlapping memory ranges, but in that case there's
      no way to tell what's desired without looking at the relocation
-     table.  */
-  if (sorted_syms[thisplace]->section != sec
-      && (aux->require_sec
-	  || ((abfd->flags & HAS_RELOC) != 0
-	      && vma >= bfd_get_section_vma (abfd, sec)
-	      && vma < (bfd_get_section_vma (abfd, sec)
-			+ bfd_section_size (abfd, sec) / opb))))
+     table.
+     
+     Also give the target a chance to reject symbols.  */
+  want_section = (aux->require_sec
+		  || ((abfd->flags & HAS_RELOC) != 0
+		      && vma >= bfd_get_section_vma (abfd, sec)
+		      && vma < (bfd_get_section_vma (abfd, sec)
+				+ bfd_section_size (abfd, sec) / opb)));
+  if ((sorted_syms[thisplace]->section != sec && want_section)
+      || !info->symbol_is_valid (sorted_syms[thisplace], info))
     {
       long i;
+      long newplace;
 
       for (i = thisplace + 1; i < sorted_symcount; i++)
 	{
@@ -751,27 +755,36 @@ find_symbol_for_address (bfd_vma vma,
 	}
 
       --i;
+      newplace = sorted_symcount;
 
       for (; i >= 0; i--)
 	{
-	  if (sorted_syms[i]->section == sec
-	      && (i == 0
-		  || sorted_syms[i - 1]->section != sec
-		  || (bfd_asymbol_value (sorted_syms[i])
-		      != bfd_asymbol_value (sorted_syms[i - 1]))))
+	  if ((sorted_syms[i]->section == sec || !want_section)
+	      && info->symbol_is_valid (sorted_syms[i], info))
 	    {
-	      thisplace = i;
-	      break;
+	      if (newplace == sorted_symcount)
+		newplace = i;
+
+	      if (bfd_asymbol_value (sorted_syms[i])
+		  != bfd_asymbol_value (sorted_syms[newplace]))
+		break;
+
+	      /* Remember this symbol and keep searching until we reach
+		 an earlier address.  */
+	      newplace = i;
 	    }
 	}
 
-      if (sorted_syms[thisplace]->section != sec)
+      if (newplace != sorted_symcount)
+	thisplace = newplace;
+      else
 	{
 	  /* We didn't find a good symbol with a smaller value.
 	     Look for one with a larger value.  */
 	  for (i = thisplace + 1; i < sorted_symcount; i++)
 	    {
-	      if (sorted_syms[i]->section == sec)
+	      if ((sorted_syms[i]->section == sec || !want_section)
+		  && info->symbol_is_valid (sorted_syms[i], info))
 		{
 		  thisplace = i;
 		  break;
@@ -779,25 +792,12 @@ find_symbol_for_address (bfd_vma vma,
 	    }
 	}
 
-      if (sorted_syms[thisplace]->section != sec
-	  && (aux->require_sec
-	      || ((abfd->flags & HAS_RELOC) != 0
-		  && vma >= bfd_get_section_vma (abfd, sec)
-		  && vma < (bfd_get_section_vma (abfd, sec)
-			    + bfd_section_size (abfd, sec)))))
+      if ((sorted_syms[thisplace]->section != sec && want_section)
+	  || !info->symbol_is_valid (sorted_syms[thisplace], info))
 	/* There is no suitable symbol.  */
 	return NULL;
     }
 
-  /* Give the target a chance to reject the symbol.  */
-  while (! info->symbol_is_valid (sorted_syms [thisplace], info))
-    {
-      ++ thisplace;
-      if (thisplace >= sorted_symcount
-	  || bfd_asymbol_value (sorted_syms [thisplace]) > vma)
-	return NULL;
-    }
-
   if (place != NULL)
     *place = thisplace;
 
@@ -1378,6 +1378,8 @@ disassemble_bytes (struct disassemble_info * info,
 	      info->flags = 0;
 
 	      if (info->disassembler_needs_relocs
+		  && (bfd_get_file_flags (aux->abfd) & EXEC_P) == 0
+		  && (bfd_get_file_flags (aux->abfd) & DYNAMIC) == 0
 		  && *relppp < relppend)
 		{
 		  bfd_signed_vma distance_to_rel;
@@ -1749,11 +1751,13 @@ disassemble_section (bfd *abfd, asection *section, void *info)
 
 	  pinfo->symbols = sorted_syms + place;
 	  pinfo->num_symbols = x - place;
+	  pinfo->symtab_pos = place;
 	}
       else
 	{
 	  pinfo->symbols = NULL;
 	  pinfo->num_symbols = 0;
+	  pinfo->symtab_pos = -1;
 	}
 
       if (! prefix_addresses)
@@ -1946,6 +1950,8 @@ disassemble_data (bfd *abfd)
 		 compare_relocs);
 	}
     }
+  disasm_info.symtab = sorted_syms;
+  disasm_info.symtab_size = sorted_symcount;
 
   bfd_map_over_sections (abfd, disassemble_section, & disasm_info);
 
@@ -1971,7 +1977,19 @@ load_debug_section (enum dwarf_section_display_enum debug, void *file)
   if (sec == NULL)
     return 0;
 
-  section->address = bfd_get_section_vma (abfd, sec);
+  /* Compute a bias to be added to offsets found within the DWARF debug
+     information.  These offsets are meant to be relative to the start of
+     the dwarf section, and hence the bias should be 0.  For MACH-O however
+     a dwarf section is really just a region of a much larger section and so
+     the bias is the address of the start of that area within the larger
+     section.  This test is important for PE and COFF based targets which
+     use DWARF debug information, since unlike ELF, they do not allow the
+     dwarf sections to be placed at address 0.  */
+  if (bfd_get_flavour (abfd) == bfd_target_mach_o_flavour)
+    section->address = bfd_get_section_vma (abfd, sec);
+  else
+    section->address = 0;
+    
   section->size = bfd_get_section_size (sec);
   section->start = xmalloc (section->size);
 
@@ -2016,7 +2034,7 @@ dump_dwarf_section (bfd *abfd, asection *section,
   const char *match;
   enum dwarf_section_display_enum i;
 
-  if (strncmp (name, ".gnu.linkonce.wi.", 17) == 0)
+  if (CONST_STRNEQ (name, ".gnu.linkonce.wi."))
     match = ".debug_info";
   else
     match = name;
@@ -2512,12 +2530,16 @@ dump_symbols (bfd *abfd ATTRIBUTE_UNUSED, bfd_boolean dynamic)
 	      /* If we want to demangle the name, we demangle it
 		 here, and temporarily clobber it while calling
 		 bfd_print_symbol.  FIXME: This is a gross hack.  */
-	      alloc = demangle (cur_bfd, name);
-	      (*current)->name = alloc;
+	      alloc = bfd_demangle (cur_bfd, name, DMGL_ANSI | DMGL_PARAMS);
+	      if (alloc != NULL)
+		(*current)->name = alloc;
 	      bfd_print_symbol (cur_bfd, stdout, *current,
 				bfd_print_symbol_all);
-	      (*current)->name = name;
-	      free (alloc);
+	      if (alloc != NULL)
+		{
+		  (*current)->name = name;
+		  free (alloc);
+		}
 	    }
 	  else
 	    bfd_print_symbol (cur_bfd, stdout, *current,
@@ -2823,8 +2845,9 @@ dump_bfd (bfd *abfd)
       dhandle = read_debugging_info (abfd, syms, symcount);
       if (dhandle != NULL)
 	{
-	  if (! print_debugging_info (stdout, dhandle, abfd, syms, demangle,
-	      dump_debugging_tags ? TRUE : FALSE))
+	  if (!print_debugging_info (stdout, dhandle, abfd, syms,
+				     bfd_demangle,
+				     dump_debugging_tags ? TRUE : FALSE))
 	    {
 	      non_fatal (_("%s: printing debugging information failed"),
 			 bfd_get_filename (abfd));
@@ -2903,7 +2926,10 @@ display_file (char *filename, char *target)
   bfd *arfile = NULL;
 
   if (get_file_size (filename) < 1)
-    return;
+    {
+      exit_status = 1;
+      return;
+    }
 
   file = bfd_openr (filename, target);
   if (file == NULL)
@@ -2952,7 +2978,6 @@ main (int argc, char **argv)
   int c;
   char *target = default_target;
   bfd_boolean seenflag = FALSE;
-  const char *demangler = NULL, *style = NULL;
 
 #if defined (HAVE_SETLOCALE)
 #if defined (HAVE_LC_MESSAGES)
@@ -3010,7 +3035,16 @@ main (int argc, char **argv)
 	case 'C':
 	  do_demangle = TRUE;
 	  if (optarg != NULL)
-	    style = optarg;
+	    {
+	      enum demangling_styles style;
+
+	      style = cplus_demangle_name_to_style (optarg);
+	      if (style == unknown_demangling)
+		fatal (_("unknown demangling style `%s'"),
+		       optarg);
+
+	      cplus_demangle_set_style (style);
+	    }
 	  break;
 	case 'w':
 	  wide_output = TRUE;
@@ -3128,9 +3162,6 @@ main (int argc, char **argv)
 	  dump_section_contents = TRUE;
 	  seenflag = TRUE;
 	  break;
-	case OPTION_DEMANGLER:
-	  demangler = optarg;
-	  break;
 	case 'r':
 	  dump_reloc_info = TRUE;
 	  seenflag = TRUE;
@@ -3161,8 +3192,6 @@ main (int argc, char **argv)
 	}
     }
 
-  init_demangler (style, NULL, demangler);
-
   if (show_version)
     print_version ("objdump");
 
