@@ -1,5 +1,5 @@
 /* rename.c -- rename a file, preserving symlinks.
-   Copyright 1999, 2002 Free Software Foundation, Inc.
+   Copyright 1999, 2002, 2003, 2007 Free Software Foundation, Inc.
 
    This file is part of GNU Binutils.
 
@@ -15,9 +15,10 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
+#include "sysdep.h"
 #include "bfd.h"
 #include "bucomm.h"
 
@@ -37,8 +38,8 @@
 #define O_BINARY 0
 #endif
 
-static int simple_copy
-  PARAMS ((const char *, const char *));
+#if ! defined (_WIN32) || defined (__CYGWIN32__)
+static int simple_copy (const char *, const char *);
 
 /* The number of bytes to copy at once.  */
 #define COPY_BUF 8192
@@ -47,9 +48,7 @@ static int simple_copy
    Return 0 if ok, -1 if error.  */
 
 static int
-simple_copy (from, to)
-     const char *from;
-     const char *to;
+simple_copy (const char *from, const char *to)
 {
   int fromfd, tofd, nread;
   int saved;
@@ -91,14 +90,13 @@ simple_copy (from, to)
     }
   return 0;
 }
+#endif /* __CYGWIN32__ or not _WIN32 */
 
 /* Set the times of the file DESTINATION to be the same as those in
    STATBUF.  */
 
 void
-set_times (destination, statbuf)
-     const char *destination;
-     const struct stat *statbuf;
+set_times (const char *destination, const struct stat *statbuf)
 {
   int result;
 
@@ -145,10 +143,7 @@ set_times (destination, statbuf)
    Return 0 if ok, -1 if error.  */
 
 int
-smart_rename (from, to, preserve_dates)
-     const char *from;
-     const char *to;
-     int preserve_dates;
+smart_rename (const char *from, const char *to, int preserve_dates ATTRIBUTE_UNUSED)
 {
   bfd_boolean exists;
   struct stat s;
@@ -167,14 +162,18 @@ smart_rename (from, to, preserve_dates)
   if (ret != 0)
     {
       /* We have to clean up here.  */
-
-      non_fatal (_("%s: rename: %s"), to, strerror (errno));
+      non_fatal (_("unable to rename '%s' reason: %s"), to, strerror (errno));
       unlink (from);
     }
 #else
   /* Use rename only if TO is not a symbolic link and has
-     only one hard link.  */
-  if (! exists || (!S_ISLNK (s.st_mode) && s.st_nlink == 1))
+     only one hard link, and we have permission to write to it.  */
+  if (! exists
+      || (!S_ISLNK (s.st_mode)
+	  && S_ISREG (s.st_mode)
+	  && (s.st_mode & S_IWUSR)
+	  && s.st_nlink == 1)
+      )
     {
       ret = rename (from, to);
       if (ret == 0)
@@ -201,7 +200,7 @@ smart_rename (from, to, preserve_dates)
       else
 	{
 	  /* We have to clean up here.  */
-	  non_fatal (_("%s: rename: %s"), to, strerror (errno));
+	  non_fatal (_("unable to rename '%s' reason: %s"), to, strerror (errno));
 	  unlink (from);
 	}
     }
@@ -209,7 +208,7 @@ smart_rename (from, to, preserve_dates)
     {
       ret = simple_copy (from, to);
       if (ret != 0)
-	non_fatal (_("%s: simple_copy: %s"), to, strerror (errno));
+	non_fatal (_("unable to copy file '%s' reason: %s"), to, strerror (errno));
 
       if (preserve_dates)
 	set_times (to, &s);
