@@ -1,6 +1,6 @@
 /* as.c - GAS main program.
    Copyright 1987, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
-   1999, 2000, 2001, 2002
+   1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
    Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
@@ -17,22 +17,19 @@
 
    You should have received a copy of the GNU General Public License
    along with GAS; see the file COPYING.  If not, write to the Free
-   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+   Software Foundation, 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
 /* Main program for AS; a 32-bit assembler of GNU.
- * Understands command arguments.
- * Has a few routines that don't fit in other modules because they
- * are shared.
- *
- *			bugs
- *
- * : initialisers
- *	Since no-one else says they will support them in future: I
- * don't support them now.
- */
-
-#include "ansidecl.h"
+   Understands command arguments.
+   Has a few routines that don't fit in other modules because they
+   are shared.
+  
+  			bugs
+  
+   : initialisers
+  	Since no-one else says they will support them in future: I
+   don't support them now.  */
 
 #define COMMON
 
@@ -42,10 +39,8 @@
 #include "sb.h"
 #include "macro.h"
 #include "dwarf2dbg.h"
-
-#ifdef BFD_ASSEMBLER
+#include "dw2gencfi.h"
 #include "bfdver.h"
-#endif
 
 #ifdef HAVE_ITBL_CPU
 #include "itbl-ops.h"
@@ -60,31 +55,45 @@ extern PTR sbrk ();
 #endif
 #endif
 
-static void show_usage PARAMS ((FILE *));
-static void parse_args PARAMS ((int *, char ***));
-static void dump_statistics PARAMS ((void));
-static void perform_an_assembly_pass PARAMS ((int argc, char **argv));
-static int macro_expr PARAMS ((const char *, int, sb *, int *));
+#ifdef USING_CGEN
+/* Perform any cgen specific initialisation for gas.  */
+extern void gas_cgen_begin (void);
+#endif
+
+/* Keep a record of the itbl files we read in.  */
+struct itbl_file_list
+{
+  struct itbl_file_list *next;
+  char *name;
+};
+
+/* We build a list of defsyms as we read the options, and then define
+   them after we have initialized everything.  */
+struct defsym_list
+{
+  struct defsym_list *next;
+  char *name;
+  valueT value;
+};
+
 
 /* True if a listing is wanted.  */
 int listing;
 
-/* Name of listing file.  */
-static char *listing_filename = NULL;
-
 /* Type of debugging to generate.  */
-
 enum debug_info_type debug_type = DEBUG_UNSPECIFIED;
+int use_gnu_debug_info_extensions = 0;
+
+#ifndef MD_DEBUG_FORMAT_SELECTOR
+#define MD_DEBUG_FORMAT_SELECTOR NULL
+#endif
+static enum debug_info_type (*md_debug_format_selector) (int *) = MD_DEBUG_FORMAT_SELECTOR;
 
 /* Maximum level of macro nesting.  */
 int max_macro_nest = 100;
 
 /* argv[0]  */
-char *myname;
-#ifdef BFD_ASSEMBLER
-segT reg_section, expr_section;
-segT text_section, data_section, bss_section;
-#endif
+static char * myname;
 
 /* The default obstack chunk size.  If we set this to zero, the
    obstack code will use whatever will fit in a 4096 byte block.  */
@@ -94,25 +103,26 @@ int chunksize = 0;
    Then the chunk sizes for gas and bfd will be reduced.  */
 int debug_memory = 0;
 
-/* We build a list of defsyms as we read the options, and then define
-   them after we have initialized everything.  */
+/* Enable verbose mode.  */
+int verbose = 0;
 
-struct defsym_list {
-  struct defsym_list *next;
-  char *name;
-  valueT value;
-};
+segT reg_section;
+segT expr_section;
+segT text_section;
+segT data_section;
+segT bss_section;
+
+/* Name of listing file.  */
+static char *listing_filename = NULL;
 
 static struct defsym_list *defsyms;
 
-/* Keep a record of the itbl files we read in.  */
-
-struct itbl_file_list {
-  struct itbl_file_list *next;
-  char *name;
-};
-
 static struct itbl_file_list *itbl_files;
+
+static long start_time;
+
+static int flag_macro_alternate;
+
 
 #ifdef USE_EMULATIONS
 #define EMULATION_ENVIRON "AS_EMULATION"
@@ -125,12 +135,8 @@ extern struct emulation crisaout, criself;
 static struct emulation *const emulations[] = { EMULATIONS };
 static const int n_emulations = sizeof (emulations) / sizeof (emulations[0]);
 
-static void select_emulation_mode PARAMS ((int, char **));
-
 static void
-select_emulation_mode (argc, argv)
-     int argc;
-     char **argv;
+select_emulation_mode (int argc, char **argv)
 {
   int i;
   char *p, *em = 0;
@@ -174,14 +180,14 @@ select_emulation_mode (argc, argv)
 }
 
 const char *
-default_emul_bfd_name ()
+default_emul_bfd_name (void)
 {
   abort ();
   return NULL;
 }
 
 void
-common_emul_init ()
+common_emul_init (void)
 {
   this_format = this_emulation->format;
 
@@ -203,25 +209,20 @@ common_emul_init ()
 #endif
 
 void
-print_version_id ()
+print_version_id (void)
 {
   static int printed;
+
   if (printed)
     return;
   printed = 1;
 
-#ifdef BFD_ASSEMBLER
-  fprintf (stderr, _("GNU assembler version %s (%s) using BFD version %s"),
+  fprintf (stderr, _("GNU assembler version %s (%s) using BFD version %s\n"),
 	   VERSION, TARGET_ALIAS, BFD_VERSION_STRING);
-#else
-  fprintf (stderr, _("GNU assembler version %s (%s)"), VERSION, TARGET_ALIAS);
-#endif
-  fprintf (stderr, "\n");
 }
 
 static void
-show_usage (stream)
-     FILE *stream;
+show_usage (FILE * stream)
 {
   fprintf (stream, _("Usage: %s [option...] [asmfile...]\n"), myname);
 
@@ -238,6 +239,8 @@ Options:\n\
                       	  s      include symbols\n\
                       	  =FILE  list to FILE (must be last sub-option)\n"));
 
+  fprintf (stream, _("\
+  --alternate             initially turn on alternate macro syntax\n"));
   fprintf (stream, _("\
   -D                      produce assembler debugging messages\n"));
   fprintf (stream, _("\
@@ -259,13 +262,25 @@ Options:\n\
     fprintf (stream, _("\
                           emulate output (default %s)\n"), def_em);
   }
+#endif
+#if defined OBJ_ELF || defined OBJ_MAYBE_ELF
+  fprintf (stream, _("\
+  --execstack             require executable stack for this object\n"));
+  fprintf (stream, _("\
+  --noexecstack           don't require executable stack for this object\n"));
 #endif
   fprintf (stream, _("\
   -f                      skip whitespace and comment preprocessing\n"));
   fprintf (stream, _("\
-  --gstabs                generate stabs debugging information\n"));
+  -g --gen-debug          generate debugging information\n"));
   fprintf (stream, _("\
-  --gdwarf2               generate DWARF2 debugging information\n"));
+  --gstabs                generate STABS debugging information\n"));
+  fprintf (stream, _("\
+  --gstabs+               generate STABS debug info with GNU extensions\n"));
+  fprintf (stream, _("\
+  --gdwarf-2              generate DWARF2 debugging information\n"));
+  fprintf (stream, _("\
+  --hash-size=<value>     set the hash table size close to <value>\n"));
   fprintf (stream, _("\
   --help                  show this message and exit\n"));
   fprintf (stream, _("\
@@ -289,6 +304,10 @@ Options:\n\
   fprintf (stream, _("\
   -R                      fold data section into text section\n"));
   fprintf (stream, _("\
+  --reduce-memory-overheads \n\
+                          prefer smaller memory use at the cost of longer\n\
+                          assembly times\n"));
+  fprintf (stream, _("\
   --statistics            print various measured statistics from execution\n"));
   fprintf (stream, _("\
   --strip-local-absolute  strip local absolute symbols\n"));
@@ -324,11 +343,15 @@ Options:\n\
   fprintf (stream, _("\
   --listing-cont-lines    set the maximum number of continuation lines used\n\
                           for the output data column of the listing\n"));
+  fprintf (stream, _("\
+  @FILE                   read options from FILE\n")); 
 
   md_show_usage (stream);
 
   fputc ('\n', stream);
-  fprintf (stream, _("Report bugs to %s\n"), REPORT_BUGS_TO);
+
+  if (REPORT_BUGS_TO[0] && stream == stdout)
+    fprintf (stream, _("Report bugs to %s\n"), REPORT_BUGS_TO);
 }
 
 /* Since it is easy to do here we interpret the special arg "-"
@@ -341,111 +364,142 @@ Options:\n\
    md_parse_option definitions in config/tc-*.c.  */
 
 static void
-parse_args (pargc, pargv)
-     int *pargc;
-     char ***pargv;
+parse_args (int * pargc, char *** pargv)
 {
-  int old_argc, new_argc;
-  char **old_argv, **new_argv;
-
+  int old_argc;
+  int new_argc;
+  char ** old_argv;
+  char ** new_argv;
   /* Starting the short option string with '-' is for programs that
      expect options and other ARGV-elements in any order and that care about
      the ordering of the two.  We describe each non-option ARGV-element
      as if it were the argument of an option with character code 1.  */
-
   char *shortopts;
   extern const char *md_shortopts;
-  static const char std_shortopts[] = {
+  static const char std_shortopts[] =
+  {
     '-', 'J',
 #ifndef WORKING_DOT_WORD
     /* -K is not meaningful if .word is not being hacked.  */
     'K',
 #endif
-    'L', 'M', 'R', 'W', 'Z', 'f', 'a', ':', ':', 'D', 'I', ':', 'o', ':',
+    'L', 'M', 'R', 'W', 'Z', 'a', ':', ':', 'D', 'f', 'g', ':',':', 'I', ':', 'o', ':',
 #ifndef VMS
     /* -v takes an argument on VMS, so we don't make it a generic
        option.  */
     'v',
 #endif
     'w', 'X',
-    /* New option for extending instruction set (see also --itbl below)  */
+    /* New option for extending instruction set (see also --itbl below).  */
     't', ':',
     '\0'
   };
   struct option *longopts;
   extern struct option md_longopts[];
   extern size_t md_longopts_size;
-  static const struct option std_longopts[] = {
-#define OPTION_HELP (OPTION_STD_BASE)
-    {"help", no_argument, NULL, OPTION_HELP},
-    /* getopt allows abbreviations, so we do this to stop it from
-       treating -k as an abbreviation for --keep-locals.  Some
-       ports use -k to enable PIC assembly.  */
-    {"keep-locals", no_argument, NULL, 'L'},
-    {"keep-locals", no_argument, NULL, 'L'},
-    {"mri", no_argument, NULL, 'M'},
-#define OPTION_NOCPP (OPTION_STD_BASE + 1)
-    {"nocpp", no_argument, NULL, OPTION_NOCPP},
-#define OPTION_STATISTICS (OPTION_STD_BASE + 2)
-    {"statistics", no_argument, NULL, OPTION_STATISTICS},
-#define OPTION_VERSION (OPTION_STD_BASE + 3)
-    {"version", no_argument, NULL, OPTION_VERSION},
-#define OPTION_DUMPCONFIG (OPTION_STD_BASE + 4)
-    {"dump-config", no_argument, NULL, OPTION_DUMPCONFIG},
-#define OPTION_VERBOSE (OPTION_STD_BASE + 5)
-    {"verbose", no_argument, NULL, OPTION_VERBOSE},
-#define OPTION_EMULATION (OPTION_STD_BASE + 6)
-    {"emulation", required_argument, NULL, OPTION_EMULATION},
-#define OPTION_DEFSYM (OPTION_STD_BASE + 7)
-    {"defsym", required_argument, NULL, OPTION_DEFSYM},
-#define OPTION_INSTTBL (OPTION_STD_BASE + 8)
+  /* Codes used for the long options with no short synonyms.  */
+  enum option_values
+    {
+      OPTION_HELP = OPTION_STD_BASE,
+      OPTION_NOCPP,
+      OPTION_STATISTICS,
+      OPTION_VERSION,
+      OPTION_DUMPCONFIG,
+      OPTION_VERBOSE,
+      OPTION_EMULATION,
+      OPTION_DEFSYM,
+      OPTION_INSTTBL,
+      OPTION_LISTING_LHS_WIDTH,
+      OPTION_LISTING_LHS_WIDTH2,
+      OPTION_LISTING_RHS_WIDTH,
+      OPTION_LISTING_CONT_LINES,
+      OPTION_DEPFILE,
+      OPTION_GSTABS,
+      OPTION_GSTABS_PLUS,
+      OPTION_GDWARF2,
+      OPTION_STRIP_LOCAL_ABSOLUTE,
+      OPTION_TRADITIONAL_FORMAT,
+      OPTION_WARN,
+      OPTION_TARGET_HELP,
+      OPTION_EXECSTACK,
+      OPTION_NOEXECSTACK,
+      OPTION_ALTERNATE,
+      OPTION_AL,
+      OPTION_HASH_TABLE_SIZE,
+      OPTION_REDUCE_MEMORY_OVERHEADS,
+      OPTION_WARN_FATAL
+    /* When you add options here, check that they do
+       not collide with OPTION_MD_BASE.  See as.h.  */
+    };
+  
+  static const struct option std_longopts[] =
+  {
+    /* Note: commas are placed at the start of the line rather than
+       the end of the preceeding line so that it is simpler to
+       selectively add and remove lines from this list.  */
+    {"alternate", no_argument, NULL, OPTION_ALTERNATE}
+    /* The entry for "a" is here to prevent getopt_long_only() from
+       considering that -a is an abbreviation for --alternate.  This is
+       necessary because -a=<FILE> is a valid switch but getopt would
+       normally reject it since --alternate does not take an argument.  */
+    ,{"a", optional_argument, NULL, 'a'}
+    /* Handle -al=<FILE>.  */
+    ,{"al", optional_argument, NULL, OPTION_AL}
+    ,{"defsym", required_argument, NULL, OPTION_DEFSYM}
+    ,{"dump-config", no_argument, NULL, OPTION_DUMPCONFIG}
+    ,{"emulation", required_argument, NULL, OPTION_EMULATION}
+#if defined OBJ_ELF || defined OBJ_MAYBE_ELF
+    ,{"execstack", no_argument, NULL, OPTION_EXECSTACK}
+    ,{"noexecstack", no_argument, NULL, OPTION_NOEXECSTACK}
+#endif
+    ,{"fatal-warnings", no_argument, NULL, OPTION_WARN_FATAL}
+    ,{"gdwarf-2", no_argument, NULL, OPTION_GDWARF2}
+    /* GCC uses --gdwarf-2 but GAS uses to use --gdwarf2,
+       so we keep it here for backwards compatibility.  */
+    ,{"gdwarf2", no_argument, NULL, OPTION_GDWARF2}
+    ,{"gen-debug", no_argument, NULL, 'g'}
+    ,{"gstabs", no_argument, NULL, OPTION_GSTABS}
+    ,{"gstabs+", no_argument, NULL, OPTION_GSTABS_PLUS}
+    ,{"hash-size", required_argument, NULL, OPTION_HASH_TABLE_SIZE}
+    ,{"help", no_argument, NULL, OPTION_HELP}
     /* New option for extending instruction set (see also -t above).
        The "-t file" or "--itbl file" option extends the basic set of
        valid instructions by reading "file", a text file containing a
        list of instruction formats.  The additional opcodes and their
        formats are added to the built-in set of instructions, and
        mnemonics for new registers may also be defined.  */
-    {"itbl", required_argument, NULL, OPTION_INSTTBL},
-#define OPTION_LISTING_LHS_WIDTH (OPTION_STD_BASE + 9)
-    {"listing-lhs-width", required_argument, NULL, OPTION_LISTING_LHS_WIDTH},
-#define OPTION_LISTING_LHS_WIDTH2 (OPTION_STD_BASE + 10)
-    {"listing-lhs-width2", required_argument, NULL, OPTION_LISTING_LHS_WIDTH2},
-#define OPTION_LISTING_RHS_WIDTH (OPTION_STD_BASE + 11)
-    {"listing-rhs-width", required_argument, NULL, OPTION_LISTING_RHS_WIDTH},
-#define OPTION_LISTING_CONT_LINES (OPTION_STD_BASE + 12)
-    {"listing-cont-lines", required_argument, NULL, OPTION_LISTING_CONT_LINES},
-#define OPTION_DEPFILE (OPTION_STD_BASE + 13)
-    {"MD", required_argument, NULL, OPTION_DEPFILE},
-#define OPTION_GSTABS (OPTION_STD_BASE + 14)
-    {"gstabs", no_argument, NULL, OPTION_GSTABS},
-#define OPTION_STRIP_LOCAL_ABSOLUTE (OPTION_STD_BASE + 15)
-    {"strip-local-absolute", no_argument, NULL, OPTION_STRIP_LOCAL_ABSOLUTE},
-#define OPTION_TRADITIONAL_FORMAT (OPTION_STD_BASE + 16)
-    {"traditional-format", no_argument, NULL, OPTION_TRADITIONAL_FORMAT},
-#define OPTION_GDWARF2 (OPTION_STD_BASE + 17)
-    {"gdwarf2", no_argument, NULL, OPTION_GDWARF2},
-    {"no-warn", no_argument, NULL, 'W'},
-#define OPTION_WARN (OPTION_STD_BASE + 18)
-    {"warn", no_argument, NULL, OPTION_WARN},
-#define OPTION_TARGET_HELP (OPTION_STD_BASE + 19)
-    {"target-help", no_argument, NULL, OPTION_TARGET_HELP},
-#define OPTION_WARN_FATAL (OPTION_STD_BASE + 20)
-    {"fatal-warnings", no_argument, NULL, OPTION_WARN_FATAL}
-    /* When you add options here, check that they do not collide with
-       OPTION_MD_BASE.  See as.h.  */
+    ,{"itbl", required_argument, NULL, OPTION_INSTTBL}
+    /* getopt allows abbreviations, so we do this to stop it from
+       treating -k as an abbreviation for --keep-locals.  Some
+       ports use -k to enable PIC assembly.  */
+    ,{"keep-locals", no_argument, NULL, 'L'}
+    ,{"keep-locals", no_argument, NULL, 'L'}
+    ,{"listing-lhs-width", required_argument, NULL, OPTION_LISTING_LHS_WIDTH}
+    ,{"listing-lhs-width2", required_argument, NULL, OPTION_LISTING_LHS_WIDTH2}
+    ,{"listing-rhs-width", required_argument, NULL, OPTION_LISTING_RHS_WIDTH}
+    ,{"listing-cont-lines", required_argument, NULL, OPTION_LISTING_CONT_LINES}
+    ,{"MD", required_argument, NULL, OPTION_DEPFILE}
+    ,{"mri", no_argument, NULL, 'M'}
+    ,{"nocpp", no_argument, NULL, OPTION_NOCPP}
+    ,{"no-warn", no_argument, NULL, 'W'}
+    ,{"reduce-memory-overheads", no_argument, NULL, OPTION_REDUCE_MEMORY_OVERHEADS}
+    ,{"statistics", no_argument, NULL, OPTION_STATISTICS}
+    ,{"strip-local-absolute", no_argument, NULL, OPTION_STRIP_LOCAL_ABSOLUTE}
+    ,{"version", no_argument, NULL, OPTION_VERSION}
+    ,{"verbose", no_argument, NULL, OPTION_VERBOSE}
+    ,{"target-help", no_argument, NULL, OPTION_TARGET_HELP}
+    ,{"traditional-format", no_argument, NULL, OPTION_TRADITIONAL_FORMAT}
+    ,{"warn", no_argument, NULL, OPTION_WARN}
   };
 
   /* Construct the option lists from the standard list and the target
      dependent list.  Include space for an extra NULL option and
      always NULL terminate.  */
   shortopts = concat (std_shortopts, md_shortopts, (char *) NULL);
-  longopts = (struct option *) xmalloc (sizeof (std_longopts)
-					+ md_longopts_size
-					+ sizeof (struct option));
+  longopts = xmalloc (sizeof (std_longopts) + md_longopts_size + sizeof (struct option));
   memcpy (longopts, std_longopts, sizeof (std_longopts));
-  memcpy ((char *) longopts + sizeof (std_longopts),
-	  md_longopts, md_longopts_size);
-  memset ((char *) longopts + sizeof (std_longopts) + md_longopts_size,
+  memcpy (((char *) longopts) + sizeof (std_longopts), md_longopts, md_longopts_size);
+  memset (((char *) longopts) + sizeof (std_longopts) + md_longopts_size,
 	  0, sizeof (struct option));
 
   /* Make a local copy of the old argv.  */
@@ -453,7 +507,7 @@ parse_args (pargc, pargv)
   old_argv = *pargv;
 
   /* Initialize a new argv that contains no options.  */
-  new_argv = (char **) xmalloc (sizeof (char *) * (old_argc + 1));
+  new_argv = xmalloc (sizeof (char *) * (old_argc + 1));
   new_argv[0] = old_argv[0];
   new_argc = 1;
   new_argv[new_argc] = NULL;
@@ -477,7 +531,7 @@ parse_args (pargc, pargv)
 	  if (md_parse_option (optc, optarg) != 0)
 	    break;
 	  /* `-v' isn't included in the general short_opts list, so check for
-	     it explicity here before deciding we've gotten a bad argument.  */
+	     it explicitly here before deciding we've gotten a bad argument.  */
 	  if (optc == 'v')
 	    {
 #ifdef VMS
@@ -493,8 +547,11 @@ parse_args (pargc, pargv)
 #endif
 	      case OPTION_VERBOSE:
 		print_version_id ();
+		verbose = 1;
 	      break;
 	    }
+	  else
+	    as_bad (_("unrecognized option -%c%s"), optc, optarg ? optarg : "");
 	  /* Fall through.  */
 
 	case '?':
@@ -532,12 +589,8 @@ parse_args (pargc, pargv)
 
 	case OPTION_VERSION:
 	  /* This output is intended to follow the GNU standards document.  */
-#ifdef BFD_ASSEMBLER
 	  printf (_("GNU assembler %s\n"), BFD_VERSION_STRING);
-#else
-	  printf (_("GNU assembler %s\n"), VERSION);
-#endif
-	  printf (_("Copyright 2002 Free Software Foundation, Inc.\n"));
+	  printf (_("Copyright 2007 Free Software Foundation, Inc.\n"));
 	  printf (_("\
 This program is free software; you may redistribute it under the terms of\n\
 the GNU General Public License.  This program has absolutely no warranty.\n"));
@@ -577,12 +630,8 @@ the GNU General Public License.  This program has absolutely no warranty.\n"));
 	    if (*s == '\0')
 	      as_fatal (_("bad defsym; format is --defsym name=value"));
 	    *s++ = '\0';
-#ifdef BFD_ASSEMBLER
 	    i = bfd_scan_vma (s, (const char **) NULL, 0);
-#else
-	    i = strtol (s, (char **) NULL, 0);
-#endif
-	    n = (struct defsym_list *) xmalloc (sizeof *n);
+	    n = xmalloc (sizeof *n);
 	    n->next = defsyms;
 	    n->name = optarg;
 	    n->value = i;
@@ -603,7 +652,7 @@ the GNU General Public License.  This program has absolutely no warranty.\n"));
 		break;
 	      }
 
-	    n = (struct itbl_file_list *) xmalloc (sizeof *n);
+	    n = xmalloc (sizeof * n);
 	    n->next = itbl_files;
 	    n->name = optarg;
 	    itbl_files = n;
@@ -623,6 +672,25 @@ the GNU General Public License.  This program has absolutely no warranty.\n"));
 	  start_dependencies (optarg);
 	  break;
 
+	case 'g':
+	  /* Some backends, eg Alpha and Mips, use the -g switch for their
+	     own purposes.  So we check here for an explicit -g and allow
+	     the backend to decide if it wants to process it.  */
+	  if (   old_argv[optind - 1][1] == 'g'
+	      && md_parse_option (optc, optarg))
+	    continue;
+
+	  if (md_debug_format_selector)
+	    debug_type = md_debug_format_selector (& use_gnu_debug_info_extensions);
+	  else if (IS_ELF)
+	    debug_type = DEBUG_DWARF2;
+	  else
+	    debug_type = DEBUG_STABS;
+	  break;
+
+	case OPTION_GSTABS_PLUS:
+	  use_gnu_debug_info_extensions = 1;
+	  /* Fall through.  */
 	case OPTION_GSTABS:
 	  debug_type = DEBUG_STABS;
 	  break;
@@ -640,7 +708,6 @@ the GNU General Public License.  This program has absolutely no warranty.\n"));
 	  flag_warn_displacement = 1;
 	  break;
 #endif
-
 	case 'L':
 	  flag_keep_locals = 1;
 	  break;
@@ -653,6 +720,7 @@ the GNU General Public License.  This program has absolutely no warranty.\n"));
 	case OPTION_LISTING_LHS_WIDTH2:
 	  {
 	    int tmp = atoi (optarg);
+
 	    if (tmp > listing_lhs_width)
 	      listing_lhs_width_second = tmp;
 	  }
@@ -689,13 +757,46 @@ the GNU General Public License.  This program has absolutely no warranty.\n"));
 	  flag_fatal_warnings = 1;
 	  break;
 
+#if defined OBJ_ELF || defined OBJ_MAYBE_ELF
+	case OPTION_EXECSTACK:
+	  flag_execstack = 1;
+	  flag_noexecstack = 0;
+	  break;
+
+	case OPTION_NOEXECSTACK:
+	  flag_noexecstack = 1;
+	  flag_execstack = 0;
+	  break;
+#endif
 	case 'Z':
 	  flag_always_generate_output = 1;
 	  break;
 
+ 	case OPTION_AL:
+	  listing |= LISTING_LISTING;
+	  if (optarg)
+	    listing_filename = xstrdup (optarg);
+	  break;
+
+ 	case OPTION_ALTERNATE:
+ 	  optarg = old_argv [optind - 1];
+ 	  while (* optarg == '-')
+ 	    optarg ++;
+
+ 	  if (strcmp (optarg, "alternate") == 0)
+ 	    {
+ 	      flag_macro_alternate = 1;
+ 	      break;
+ 	    }
+ 	  optarg ++;
+ 	  /* Fall through.  */
+
 	case 'a':
 	  if (optarg)
 	    {
+	      if (optarg != old_argv[optind] && optarg[-1] == '=')
+		--optarg;
+
 	      if (md_parse_option (optc, optarg) != 0)
 		break;
 
@@ -752,6 +853,7 @@ the GNU General Public License.  This program has absolutely no warranty.\n"));
 	case 'I':
 	  {			/* Include file directory.  */
 	    char *temp = xstrdup (optarg);
+
 	    add_include_dir (temp);
 	    break;
 	  }
@@ -766,6 +868,24 @@ the GNU General Public License.  This program has absolutely no warranty.\n"));
 	case 'X':
 	  /* -X means treat warnings as errors.  */
 	  break;
+
+	case OPTION_REDUCE_MEMORY_OVERHEADS:
+	  /* The only change we make at the moment is to reduce
+	     the size of the hash tables that we use.  */
+	  set_gas_hash_table_size (4051);
+	  break;
+
+	case OPTION_HASH_TABLE_SIZE:
+	  {
+	    unsigned long new_size;
+
+            new_size = strtoul (optarg, NULL, 0);
+            if (new_size)
+              set_gas_hash_table_size (new_size);
+            else
+              as_fatal (_("--hash-size needs a numeric argument"));
+	    break;
+	  }
 	}
     }
 
@@ -780,16 +900,141 @@ the GNU General Public License.  This program has absolutely no warranty.\n"));
 #endif
 }
 
-static long start_time;
+static void
+dump_statistics (void)
+{
+#ifdef HAVE_SBRK
+  char *lim = (char *) sbrk (0);
+#endif
+  long run_time = get_run_time () - start_time;
 
-int main PARAMS ((int, char **));
+  fprintf (stderr, _("%s: total time in assembly: %ld.%06ld\n"),
+	   myname, run_time / 1000000, run_time % 1000000);
+#ifdef HAVE_SBRK
+  fprintf (stderr, _("%s: data size %ld\n"),
+	   myname, (long) (lim - (char *) &environ));
+#endif
+
+  subsegs_print_statistics (stderr);
+  write_print_statistics (stderr);
+  symbol_print_statistics (stderr);
+  read_print_statistics (stderr);
+
+#ifdef tc_print_statistics
+  tc_print_statistics (stderr);
+#endif
+
+#ifdef obj_print_statistics
+  obj_print_statistics (stderr);
+#endif
+}
+
+#ifndef OBJ_VMS
+static void
+close_output_file (void)
+{
+  output_file_close (out_file_name);
+}
+#endif
+
+/* The interface between the macro code and gas expression handling.  */
+
+static int
+macro_expr (const char *emsg, int idx, sb *in, int *val)
+{
+  char *hold;
+  expressionS ex;
+
+  sb_terminate (in);
+
+  hold = input_line_pointer;
+  input_line_pointer = in->ptr + idx;
+  expression_and_evaluate (&ex);
+  idx = input_line_pointer - in->ptr;
+  input_line_pointer = hold;
+
+  if (ex.X_op != O_constant)
+    as_bad ("%s", emsg);
+
+  *val = (int) ex.X_add_number;
+
+  return idx;
+}
+
+/* Here to attempt 1 pass over each input file.
+   We scan argv[*] looking for filenames or exactly "" which is
+   shorthand for stdin. Any argv that is NULL is not a file-name.
+   We set need_pass_2 TRUE if, after this, we still have unresolved
+   expressions of the form (unknown value)+-(unknown value).
+
+   Note the un*x semantics: there is only 1 logical input file, but it
+   may be a catenation of many 'physical' input files.  */
+
+static void
+perform_an_assembly_pass (int argc, char ** argv)
+{
+  int saw_a_file = 0;
+  flagword applicable;
+
+  need_pass_2 = 0;
+
+  /* Create the standard sections, and those the assembler uses
+     internally.  */
+  text_section = subseg_new (TEXT_SECTION_NAME, 0);
+  data_section = subseg_new (DATA_SECTION_NAME, 0);
+  bss_section = subseg_new (BSS_SECTION_NAME, 0);
+  /* @@ FIXME -- we're setting the RELOC flag so that sections are assumed
+     to have relocs, otherwise we don't find out in time.  */
+  applicable = bfd_applicable_section_flags (stdoutput);
+  bfd_set_section_flags (stdoutput, text_section,
+			 applicable & (SEC_ALLOC | SEC_LOAD | SEC_RELOC
+				       | SEC_CODE | SEC_READONLY));
+  bfd_set_section_flags (stdoutput, data_section,
+			 applicable & (SEC_ALLOC | SEC_LOAD | SEC_RELOC
+				       | SEC_DATA));
+  bfd_set_section_flags (stdoutput, bss_section, applicable & SEC_ALLOC);
+  seg_info (bss_section)->bss = 1;
+  subseg_new (BFD_ABS_SECTION_NAME, 0);
+  subseg_new (BFD_UND_SECTION_NAME, 0);
+  reg_section = subseg_new ("*GAS `reg' section*", 0);
+  expr_section = subseg_new ("*GAS `expr' section*", 0);
+
+  subseg_set (text_section, 0);
+
+  /* This may add symbol table entries, which requires having an open BFD,
+     and sections already created.  */
+  md_begin ();
+
+#ifdef USING_CGEN
+  gas_cgen_begin ();
+#endif
+#ifdef obj_begin
+  obj_begin ();
+#endif
+
+  /* Skip argv[0].  */
+  argv++;
+  argc--;
+
+  while (argc--)
+    {
+      if (*argv)
+	{			/* Is it a file-name argument?  */
+	  PROGRESS (1);
+	  saw_a_file++;
+	  /* argv->"" if stdin desired, else->filename.  */
+	  read_a_source_file (*argv);
+	}
+      argv++;			/* Completed that argv.  */
+    }
+  if (!saw_a_file)
+    read_a_source_file ("");
+}
+
 
 int
-main (argc, argv)
-     int argc;
-     char **argv;
+main (int argc, char ** argv)
 {
-  int macro_alternate;
   int macro_strip_at;
   int keep_it;
 
@@ -814,6 +1059,8 @@ main (argc, argv)
   myname = argv[0];
   xmalloc_set_program_name (myname);
 
+  expandargv (&argc, &argv);
+
   START_PROGRESS (myname, 0);
 
 #ifndef OBJ_DEFAULT_OUTPUT_FILE_NAME
@@ -823,47 +1070,43 @@ main (argc, argv)
   out_file_name = OBJ_DEFAULT_OUTPUT_FILE_NAME;
 
   hex_init ();
-#ifdef BFD_ASSEMBLER
   bfd_init ();
   bfd_set_error_program_name (myname);
-#endif
 
 #ifdef USE_EMULATIONS
   select_emulation_mode (argc, argv);
 #endif
 
   PROGRESS (1);
+  /* Call parse_args before any of the init/begin functions
+     so that switches like --hash-size can be honored.  */
+  parse_args (&argc, &argv);
   symbol_begin ();
   frag_init ();
   subsegs_begin ();
-  parse_args (&argc, &argv);
   read_begin ();
   input_scrub_begin ();
   expr_begin ();
 
+#ifndef OBJ_VMS /* Does its own file handling.  */
+  /* It has to be called after dump_statistics ().  */
+  xatexit (close_output_file);
+#endif
+
   if (flag_print_statistics)
     xatexit (dump_statistics);
 
-  macro_alternate = 0;
   macro_strip_at = 0;
 #ifdef TC_I960
   macro_strip_at = flag_mri;
 #endif
-#ifdef TC_A29K
-  /* For compatibility with the AMD 29K family macro assembler
-     specification.  */
-  macro_alternate = 1;
-  macro_strip_at = 1;
-#endif
 
-  macro_init (macro_alternate, flag_mri, macro_strip_at, macro_expr);
+  macro_init (flag_macro_alternate, flag_mri, macro_strip_at, macro_expr);
 
   PROGRESS (1);
 
-#ifdef BFD_ASSEMBLER
   output_file_create (out_file_name);
   assert (stdoutput != 0);
-#endif
 
 #ifdef tc_init_after_args
   tc_init_after_args ();
@@ -881,6 +1124,11 @@ main (argc, argv)
 
       sym = symbol_new (defsyms->name, absolute_section, defsyms->value,
 			&zero_address_frag);
+      /* Make symbols defined on the command line volatile, so that they
+	 can be redefined inside a source file.  This makes this assembler's
+	 behaviour compatible with earlier versions, but it may not be
+	 completely intuitive.  */
+      S_SET_VOLATILE (sym);
       symbol_table_insert (sym);
       next = defsyms->next;
       free (defsyms);
@@ -898,23 +1146,38 @@ main (argc, argv)
   md_end ();
 #endif
 
+#if defined OBJ_ELF || defined OBJ_MAYBE_ELF
+  if ((flag_execstack || flag_noexecstack)
+      && OUTPUT_FLAVOR == bfd_target_elf_flavour)
+    {
+      segT gnustack;
+
+      gnustack = subseg_new (".note.GNU-stack", 0);
+      bfd_set_section_flags (stdoutput, gnustack,
+			     SEC_READONLY | (flag_execstack ? SEC_CODE : 0));
+                                                                             
+    }
+#endif
+
   /* If we've been collecting dwarf2 .debug_line info, either for
      assembly debugging or on behalf of the compiler, emit it now.  */
   dwarf2_finish ();
 
+  /* If we constructed dwarf2 .eh_frame info, either via .cfi 
+     directives from the user or by the backend, emit it now.  */
+  cfi_finish ();
+
   if (seen_at_least_1_file ()
       && (flag_always_generate_output || had_errors () == 0))
     keep_it = 1;
   else
     keep_it = 0;
 
-#if defined (BFD_ASSEMBLER) || !defined (BFD)
   /* This used to be done at the start of write_object_file in
      write.c, but that caused problems when doing listings when
      keep_it was zero.  This could probably be moved above md_end, but
      I didn't want to risk the change.  */
   subsegs_finish ();
-#endif
 
   if (keep_it)
     write_object_file ();
@@ -923,13 +1186,6 @@ main (argc, argv)
   listing_print (listing_filename);
 #endif
 
-#ifndef OBJ_VMS /* does its own file handling */
-#ifndef BFD_ASSEMBLER
-  if (keep_it)
-#endif
-    output_file_close (out_file_name);
-#endif
-
   if (flag_fatal_warnings && had_warnings () > 0 && had_errors () == 0)
     as_bad (_("%d warnings, treating warnings as errors"), had_warnings ());
 
@@ -937,7 +1193,7 @@ main (argc, argv)
     keep_it = 0;
 
   if (!keep_it)
-    unlink (out_file_name);
+    unlink_if_ordinary (out_file_name);
 
   input_scrub_end ();
 
@@ -953,164 +1209,3 @@ main (argc, argv)
 
   xexit (EXIT_SUCCESS);
 }
-
-static void
-dump_statistics ()
-{
-#ifdef HAVE_SBRK
-  char *lim = (char *) sbrk (0);
-#endif
-  long run_time = get_run_time () - start_time;
-
-  fprintf (stderr, _("%s: total time in assembly: %ld.%06ld\n"),
-	   myname, run_time / 1000000, run_time % 1000000);
-#ifdef HAVE_SBRK
-  fprintf (stderr, _("%s: data size %ld\n"),
-	   myname, (long) (lim - (char *) &environ));
-#endif
-
-  subsegs_print_statistics (stderr);
-  write_print_statistics (stderr);
-  symbol_print_statistics (stderr);
-  read_print_statistics (stderr);
-
-#ifdef tc_print_statistics
-  tc_print_statistics (stderr);
-#endif
-#ifdef obj_print_statistics
-  obj_print_statistics (stderr);
-#endif
-}
-
-/* Here to attempt 1 pass over each input file.
-   We scan argv[*] looking for filenames or exactly "" which is
-   shorthand for stdin. Any argv that is NULL is not a file-name.
-   We set need_pass_2 TRUE if, after this, we still have unresolved
-   expressions of the form (unknown value)+-(unknown value).
-
-   Note the un*x semantics: there is only 1 logical input file, but it
-   may be a catenation of many 'physical' input files.  */
-
-static void
-perform_an_assembly_pass (argc, argv)
-     int argc;
-     char **argv;
-{
-  int saw_a_file = 0;
-#ifdef BFD_ASSEMBLER
-  flagword applicable;
-#endif
-
-  need_pass_2 = 0;
-
-#ifndef BFD_ASSEMBLER
-#ifdef MANY_SEGMENTS
-  {
-    unsigned int i;
-    for (i = SEG_E0; i < SEG_UNKNOWN; i++)
-      segment_info[i].fix_root = 0;
-  }
-  /* Create the three fixed ones.  */
-  {
-    segT seg;
-
-#ifdef TE_APOLLO
-    seg = subseg_new (".wtext", 0);
-#else
-    seg = subseg_new (".text", 0);
-#endif
-    assert (seg == SEG_E0);
-    seg = subseg_new (".data", 0);
-    assert (seg == SEG_E1);
-    seg = subseg_new (".bss", 0);
-    assert (seg == SEG_E2);
-#ifdef TE_APOLLO
-    create_target_segments ();
-#endif
-  }
-
-#else /* not MANY_SEGMENTS */
-  text_fix_root = NULL;
-  data_fix_root = NULL;
-  bss_fix_root = NULL;
-#endif /* not MANY_SEGMENTS */
-#else /* BFD_ASSEMBLER */
-  /* Create the standard sections, and those the assembler uses
-     internally.  */
-  text_section = subseg_new (TEXT_SECTION_NAME, 0);
-  data_section = subseg_new (DATA_SECTION_NAME, 0);
-  bss_section = subseg_new (BSS_SECTION_NAME, 0);
-  /* @@ FIXME -- we're setting the RELOC flag so that sections are assumed
-     to have relocs, otherwise we don't find out in time.  */
-  applicable = bfd_applicable_section_flags (stdoutput);
-  bfd_set_section_flags (stdoutput, text_section,
-			 applicable & (SEC_ALLOC | SEC_LOAD | SEC_RELOC
-				       | SEC_CODE | SEC_READONLY));
-  bfd_set_section_flags (stdoutput, data_section,
-			 applicable & (SEC_ALLOC | SEC_LOAD | SEC_RELOC
-				       | SEC_DATA));
-  bfd_set_section_flags (stdoutput, bss_section, applicable & SEC_ALLOC);
-  seg_info (bss_section)->bss = 1;
-  subseg_new (BFD_ABS_SECTION_NAME, 0);
-  subseg_new (BFD_UND_SECTION_NAME, 0);
-  reg_section = subseg_new ("*GAS `reg' section*", 0);
-  expr_section = subseg_new ("*GAS `expr' section*", 0);
-
-#endif /* BFD_ASSEMBLER */
-
-  subseg_set (text_section, 0);
-
-  /* This may add symbol table entries, which requires having an open BFD,
-     and sections already created, in BFD_ASSEMBLER mode.  */
-  md_begin ();
-
-#ifdef obj_begin
-  obj_begin ();
-#endif
-
-  /* Skip argv[0].  */
-  argv++;
-  argc--;
-
-  while (argc--)
-    {
-      if (*argv)
-	{			/* Is it a file-name argument?  */
-	  PROGRESS (1);
-	  saw_a_file++;
-	  /* argv->"" if stdin desired, else->filename  */
-	  read_a_source_file (*argv);
-	}
-      argv++;			/* completed that argv  */
-    }
-  if (!saw_a_file)
-    read_a_source_file ("");
-}
-
-/* The interface between the macro code and gas expression handling.  */
-
-static int
-macro_expr (emsg, idx, in, val)
-     const char *emsg;
-     int idx;
-     sb *in;
-     int *val;
-{
-  char *hold;
-  expressionS ex;
-
-  sb_terminate (in);
-
-  hold = input_line_pointer;
-  input_line_pointer = in->ptr + idx;
-  expression (&ex);
-  idx = input_line_pointer - in->ptr;
-  input_line_pointer = hold;
-
-  if (ex.X_op != O_constant)
-    as_bad ("%s", emsg);
-
-  *val = (int) ex.X_add_number;
-
-  return idx;
-}
