@@ -1,6 +1,6 @@
 /* ECOFF object file format.
-   Copyright 1993, 1994, 1995, 1996, 1998, 1999, 2000, 2001
-   Free Software Foundation, Inc.
+   Copyright 1993, 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002,
+   2005  Free Software Foundation, Inc.
    Contributed by Cygnus Support.
    This file was put together by Ian Lance Taylor <ian@cygnus.com>.
 
@@ -18,8 +18,8 @@
 
    You should have received a copy of the GNU General Public License
    along with GAS; see the file COPYING.  If not, write to the Free
-   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+   Software Foundation, 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
 #define OBJ_HEADER "obj-ecoff.h"
 #include "as.h"
@@ -31,10 +31,192 @@
    gas directory.  This file mostly just arranges to call that one at
    the right times.  */
 
-static int ecoff_sec_sym_ok_for_reloc PARAMS ((asection *));
-static void obj_ecoff_frob_symbol PARAMS ((symbolS *, int *));
-static void ecoff_pop_insert PARAMS ((void));
-static int ecoff_separate_stab_sections PARAMS ((void));
+/* Set section VMAs and GP values before reloc processing.  */
+
+void
+ecoff_frob_file_before_fix (void)
+{
+  bfd_vma addr;
+  asection *sec;
+
+  /* Set the section VMA values.  We force the .sdata and .sbss
+     sections to the end to ensure that their VMA addresses are close
+     together so that the GP register can address both of them.  We
+     put the .bss section after the .sbss section.
+
+     Also, for the Alpha, we must sort the sections, to make sure they
+     appear in the output file in the correct order.  (Actually, maybe
+     this is a job for BFD.  But the VMAs computed would be out of
+     whack if we computed them given our initial, random ordering.
+     It's possible that that wouldn't break things; I could do some
+     experimenting sometime and find out.
+
+     This output ordering of sections is magic, on the Alpha, at
+     least.  The .lita section must come before .lit8 and .lit4,
+     otherwise the OSF/1 linker may silently trash the .lit{4,8}
+     section contents.  Also, .text must preceed .rdata.  These differ
+     from the order described in some parts of the DEC OSF/1 Assembly
+     Language Programmer's Guide, but that order doesn't seem to work
+     with their linker.
+
+     I don't know if section ordering on the MIPS is important.  */
+
+  static const char *const names[] =
+  {
+    /* text segment */
+    ".text", ".rdata", ".init", ".fini",
+    /* data segment */
+    ".data", ".lita", ".lit8", ".lit4", ".sdata", ".got",
+    /* bss segment */
+    ".sbss", ".bss",
+  };
+#define n_names ((int) (sizeof (names) / sizeof (names[0])))
+
+  /* Sections that match names, order to be straightened out later.  */
+  asection *secs[n_names];
+  int i;
+
+  addr = 0;
+  for (i = 0; i < n_names; i++)
+    secs[i] = NULL;
+
+  for (sec = stdoutput->sections; sec != NULL; sec = sec->next)
+    {
+      for (i = 0; i < n_names; i++)
+	if (!strcmp (sec->name, names[i]))
+	  {
+	    secs[i] = sec;
+	    bfd_section_list_remove (stdoutput, sec);
+	    break;
+	  }
+      if (i == n_names)
+	{
+	  bfd_set_section_vma (stdoutput, sec, addr);
+	  addr += bfd_section_size (stdoutput, sec);
+	}
+    }
+  for (i = 0; i < n_names; i++)
+    if (secs[i])
+      {
+	bfd_set_section_vma (stdoutput, secs[i], addr);
+	addr += bfd_section_size (stdoutput, secs[i]);
+      }
+  for (i = n_names - 1; i >= 0; i--)
+    if (secs[i])
+      bfd_section_list_prepend (stdoutput, secs[i]);
+
+  /* Fill in the register masks.  */
+  {
+    unsigned long gprmask = 0;
+    unsigned long fprmask = 0;
+    unsigned long *cprmask = NULL;
+
+#ifdef TC_MIPS
+    /* Fill in the MIPS register masks.  It's probably not worth
+       setting up a generic interface for this.  */
+    gprmask = mips_gprmask;
+    cprmask = mips_cprmask;
+#endif
+
+#ifdef TC_ALPHA
+    alpha_frob_ecoff_data ();
+
+    if (! bfd_ecoff_set_gp_value (stdoutput, alpha_gp_value))
+      as_fatal (_("Can't set GP value"));
+
+    gprmask = alpha_gprmask;
+    fprmask = alpha_fprmask;
+#endif
+
+    if (! bfd_ecoff_set_regmasks (stdoutput, gprmask, fprmask, cprmask))
+      as_fatal (_("Can't set register masks"));
+  }
+}
+
+/* Swap out the symbols and debugging information for BFD.  */
+
+void
+ecoff_frob_file (void)
+{
+  const struct ecoff_debug_swap * const debug_swap
+    = &ecoff_backend (stdoutput)->debug_swap;
+  bfd_vma addr ATTRIBUTE_UNUSED;
+  HDRR *hdr;
+  char *buf;
+  char *set;
+
+  /* Build the ECOFF debugging information.  */
+  assert (ecoff_data (stdoutput) != 0);
+  hdr = &ecoff_data (stdoutput)->debug_info.symbolic_header;
+  ecoff_build_debug (hdr, &buf, debug_swap);
+
+  /* Finish up the ecoff_tdata structure.  */
+  set = buf;
+#define SET(ptr, count, type, size) \
+  if (hdr->count == 0) \
+    ecoff_data (stdoutput)->debug_info.ptr = NULL; \
+  else \
+    { \
+      ecoff_data (stdoutput)->debug_info.ptr = (type) set; \
+      set += hdr->count * size; \
+    }
+
+  SET (line, cbLine, unsigned char *, sizeof (unsigned char));
+  SET (external_dnr, idnMax, void *, debug_swap->external_dnr_size);
+  SET (external_pdr, ipdMax, void *, debug_swap->external_pdr_size);
+  SET (external_sym, isymMax, void *, debug_swap->external_sym_size);
+  SET (external_opt, ioptMax, void *, debug_swap->external_opt_size);
+  SET (external_aux, iauxMax, union aux_ext *, sizeof (union aux_ext));
+  SET (ss, issMax, char *, sizeof (char));
+  SET (ssext, issExtMax, char *, sizeof (char));
+  SET (external_rfd, crfd, void *, debug_swap->external_rfd_size);
+  SET (external_fdr, ifdMax, void *, debug_swap->external_fdr_size);
+  SET (external_ext, iextMax, void *, debug_swap->external_ext_size);
+#undef SET
+}
+
+/* This is called by the ECOFF code to set the external information
+   for a symbol.  We just pass it on to BFD, which expects the swapped
+   information to be stored in the native field of the symbol.  */
+
+void
+obj_ecoff_set_ext (symbolS *sym, EXTR *ext)
+{
+  const struct ecoff_debug_swap * const debug_swap
+    = &ecoff_backend (stdoutput)->debug_swap;
+  ecoff_symbol_type *esym;
+
+  know (bfd_asymbol_flavour (symbol_get_bfdsym (sym))
+	== bfd_target_ecoff_flavour);
+  esym = ecoffsymbol (symbol_get_bfdsym (sym));
+  esym->local = FALSE;
+  esym->native = xmalloc (debug_swap->external_ext_size);
+  (*debug_swap->swap_ext_out) (stdoutput, ext, esym->native);
+}
+
+static int
+ecoff_sec_sym_ok_for_reloc (asection *sec ATTRIBUTE_UNUSED)
+{
+  return 1;
+}
+
+static void
+obj_ecoff_frob_symbol (symbolS *sym, int *puntp ATTRIBUTE_UNUSED)
+{
+  ecoff_frob_symbol (sym);
+}
+
+static void
+ecoff_pop_insert (void)
+{
+  pop_insert (obj_pseudo_table);
+}
+
+static int
+ecoff_separate_stab_sections (void)
+{
+  return 0;
+}
 
 /* These are the pseudo-ops we support in this file.  Only those
    relating to debugging information are supported here.
@@ -98,220 +280,36 @@ const pseudo_typeS obj_pseudo_table[] =
   { NULL,	s_ignore,		0 }
 };
 
-/* Swap out the symbols and debugging information for BFD.  */
-
-void
-ecoff_frob_file ()
-{
-  const struct ecoff_debug_swap * const debug_swap
-    = &ecoff_backend (stdoutput)->debug_swap;
-  bfd_vma addr;
-  asection **sec;
-  HDRR *hdr;
-  char *buf;
-  char *set;
-
-  /* Set the section VMA values.  We force the .sdata and .sbss
-     sections to the end to ensure that their VMA addresses are close
-     together so that the GP register can address both of them.  We
-     put the .bss section after the .sbss section.
-
-     Also, for the Alpha, we must sort the sections, to make sure they
-     appear in the output file in the correct order.  (Actually, maybe
-     this is a job for BFD.  But the VMAs computed would be out of
-     whack if we computed them given our initial, random ordering.
-     It's possible that that wouldn't break things; I could do some
-     experimenting sometime and find out.
-
-     This output ordering of sections is magic, on the Alpha, at
-     least.  The .lita section must come before .lit8 and .lit4,
-     otherwise the OSF/1 linker may silently trash the .lit{4,8}
-     section contents.  Also, .text must preceed .rdata.  These differ
-     from the order described in some parts of the DEC OSF/1 Assembly
-     Language Programmer's Guide, but that order doesn't seem to work
-     with their linker.
-
-     I don't know if section ordering on the MIPS is important.  */
-
-  static const char *const names[] = {
-    /* text segment */
-    ".text", ".rdata", ".init", ".fini",
-    /* data segment */
-    ".data", ".lita", ".lit8", ".lit4", ".sdata", ".got",
-    /* bss segment */
-    ".sbss", ".bss",
-  };
-#define n_names ((int) (sizeof (names) / sizeof (names[0])))
-
-  /* Sections that match names, order to be straightened out later.  */
-  asection *secs[n_names];
-  int i;
-
-  addr = 0;
-  for (i = 0; i < n_names; i++)
-    secs[i] = 0;
-
-  for (sec = &stdoutput->sections; *sec != (asection *) NULL; )
-    {
-      for (i = 0; i < n_names; i++)
-	if (!strcmp ((*sec)->name, names[i]))
-	  {
-	    secs[i] = *sec;
-	    bfd_section_list_remove (stdoutput, sec);
-	    break;
-	  }
-      if (i == n_names)
-	{
-	  bfd_set_section_vma (stdoutput, *sec, addr);
-	  addr += bfd_section_size (stdoutput, *sec);
-	  sec = &(*sec)->next;
-	}
-    }
-  for (i = 0; i < n_names; i++)
-    if (secs[i])
-      {
-	bfd_set_section_vma (stdoutput, secs[i], addr);
-	addr += bfd_section_size (stdoutput, secs[i]);
-      }
-  for (i = n_names - 1; i >= 0; i--)
-    if (secs[i])
-      bfd_section_list_insert (stdoutput, &stdoutput->sections, secs[i]);
-
-  /* Build the ECOFF debugging information.  */
-  assert (ecoff_data (stdoutput) != 0);
-  hdr = &ecoff_data (stdoutput)->debug_info.symbolic_header;
-  ecoff_build_debug (hdr, &buf, debug_swap);
-
-  /* Finish up the ecoff_tdata structure.  */
-  set = buf;
-#define SET(ptr, count, type, size) \
-  if (hdr->count == 0) \
-    ecoff_data (stdoutput)->debug_info.ptr = (type) NULL; \
-  else \
-    { \
-      ecoff_data (stdoutput)->debug_info.ptr = (type) set; \
-      set += hdr->count * size; \
-    }
-
-  SET (line, cbLine, unsigned char *, sizeof (unsigned char));
-  SET (external_dnr, idnMax, PTR, debug_swap->external_dnr_size);
-  SET (external_pdr, ipdMax, PTR, debug_swap->external_pdr_size);
-  SET (external_sym, isymMax, PTR, debug_swap->external_sym_size);
-  SET (external_opt, ioptMax, PTR, debug_swap->external_opt_size);
-  SET (external_aux, iauxMax, union aux_ext *, sizeof (union aux_ext));
-  SET (ss, issMax, char *, sizeof (char));
-  SET (ssext, issExtMax, char *, sizeof (char));
-  SET (external_rfd, crfd, PTR, debug_swap->external_rfd_size);
-  SET (external_fdr, ifdMax, PTR, debug_swap->external_fdr_size);
-  SET (external_ext, iextMax, PTR, debug_swap->external_ext_size);
-
-#undef SET
-
-  /* Fill in the register masks.  */
-  {
-    unsigned long gprmask = 0;
-    unsigned long fprmask = 0;
-    unsigned long *cprmask = NULL;
-
-#ifdef TC_MIPS
-    /* Fill in the MIPS register masks.  It's probably not worth
-       setting up a generic interface for this.  */
-    gprmask = mips_gprmask;
-    cprmask = mips_cprmask;
-#endif
-
-#ifdef TC_ALPHA
-    alpha_frob_ecoff_data ();
-
-    if (! bfd_ecoff_set_gp_value (stdoutput, alpha_gp_value))
-      as_fatal (_("Can't set GP value"));
-
-    gprmask = alpha_gprmask;
-    fprmask = alpha_fprmask;
-#endif
-
-    if (! bfd_ecoff_set_regmasks (stdoutput, gprmask, fprmask, cprmask))
-      as_fatal (_("Can't set register masks"));
-  }
-}
-
-/* This is called by the ECOFF code to set the external information
-   for a symbol.  We just pass it on to BFD, which expects the swapped
-   information to be stored in the native field of the symbol.  */
-
-void
-obj_ecoff_set_ext (sym, ext)
-     symbolS *sym;
-     EXTR *ext;
-{
-  const struct ecoff_debug_swap * const debug_swap
-    = &ecoff_backend (stdoutput)->debug_swap;
-  ecoff_symbol_type *esym;
-
-  know (bfd_asymbol_flavour (symbol_get_bfdsym (sym))
-	== bfd_target_ecoff_flavour);
-  esym = ecoffsymbol (symbol_get_bfdsym (sym));
-  esym->local = false;
-  esym->native = xmalloc (debug_swap->external_ext_size);
-  (*debug_swap->swap_ext_out) (stdoutput, ext, esym->native);
-}
-
-static int
-ecoff_sec_sym_ok_for_reloc (sec)
-     asection *sec ATTRIBUTE_UNUSED;
-{
-  return 1;
-}
-
-static void
-obj_ecoff_frob_symbol (sym, puntp)
-     symbolS *sym;
-     int *puntp ATTRIBUTE_UNUSED;
-{
-  ecoff_frob_symbol (sym);
-}
-
-static void
-ecoff_pop_insert ()
-{
-  pop_insert (obj_pseudo_table);
-}
-
-static int
-ecoff_separate_stab_sections ()
-{
-  return 0;
-}
-
 const struct format_ops ecoff_format_ops =
 {
   bfd_target_ecoff_flavour,
-  0,	/* dfl_leading_underscore */
+  0,	/* dfl_leading_underscore.  */
 
   /* FIXME: A comment why emit_section_symbols is different here (1) from
      the single-format definition (0) would be in order.  */
-  1,	/* emit_section_symbols */
-  0,	/* begin */
+  1,	/* emit_section_symbols.  */
+  0,	/* begin.  */
   ecoff_new_file,
   obj_ecoff_frob_symbol,
   ecoff_frob_file,
-  0,	/* frob_file_before_adjust */
-  0,	/* frob_file_after_relocs */
-  0,	/* s_get_size */
-  0,	/* s_set_size */
-  0,	/* s_get_align */
-  0,	/* s_set_align */
-  0,	/* s_get_other */
-  0,	/* s_set_other */
-  0,	/* s_get_desc */
-  0,	/* s_set_desc */
-  0,	/* s_get_type */
-  0,	/* s_set_type */
-  0,	/* copy_symbol_attributes */
+  0,	/* frob_file_before_adjust.  */
+  ecoff_frob_file_before_fix,
+  0,	/* frob_file_after_relocs.  */
+  0,	/* s_get_size.  */
+  0,	/* s_set_size.  */
+  0,	/* s_get_align.  */
+  0,	/* s_set_align.  */
+  0,	/* s_get_other.  */
+  0,	/* s_set_other.  */
+  0,	/* s_get_desc.  */
+  0,	/* s_set_desc.  */
+  0,	/* s_get_type.  */
+  0,	/* s_set_type.  */
+  0,	/* copy_symbol_attributes.  */
   ecoff_generate_asm_lineno,
   ecoff_stab,
   ecoff_separate_stab_sections,
-  0,	/* init_stab_section */
+  0,	/* init_stab_section.  */
   ecoff_sec_sym_ok_for_reloc,
   ecoff_pop_insert,
   ecoff_set_ext,
