@@ -1,6 +1,6 @@
 /* This file is tc-alpha.h
    Copyright 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003
-   2005
+   2005, 2006
    Free Software Foundation, Inc.
    Written by Ken Raeburn <raeburn@cygnus.com>.
 
@@ -180,4 +180,4 @@ extern void alpha_cfi_frame_initial_instructions (void);
 
 #define DWARF2_LINE_MIN_INSN_LENGTH	4
 #define DWARF2_DEFAULT_RETURN_COLUMN	26
-#define DWARF2_CIE_DATA_ALIGNMENT	-8
+#define DWARF2_CIE_DATA_ALIGNMENT	(-8)
