@@ -52,9 +52,9 @@ extern bfd_boolean bfin_start_label PARAMS ((char *));
 #define TC_EOL_IN_INSN(PTR) (bfin_eol_in_insn(PTR) ? 1 : 0)
 extern bfd_boolean bfin_eol_in_insn PARAMS ((char *));
 
-/* The instruction is permitted to contain an = character.  */
-#define TC_EQUAL_IN_INSN(C, NAME) (bfin_name_is_register (NAME) ? 1 : 0)
-extern bfd_boolean bfin_name_is_register PARAMS ((char *));
+/* Almost all instructions of Blackfin contain an = character.  */
+#define TC_EQUAL_IN_INSN(C, NAME) 1
+
 #define NOP_OPCODE 0x0000 
 
 #define LOCAL_LABELS_FB 1
@@ -75,4 +75,7 @@ extern long md_pcrel_from_section PARAMS ((struct fix *, segT));
 /* Values passed to md_apply_fix3 don't include symbol values.  */
 #define MD_APPLY_SYM_VALUE(FIX) 0
 
+/* This target is buggy, and sets fix size too large.  */
+#define TC_FX_SIZE_SLACK(FIX) 2
+
 /* end of tc-bfin.h */
