@@ -1,6 +1,6 @@
 /* sym_ids.h
 
-   Copyright 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright 2000, 2001, 2002, 2004 Free Software Foundation, Inc.
 
 This file is part of GNU Binutils.
 
@@ -16,7 +16,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #ifndef sym_ids_h
 #define sym_ids_h
@@ -35,8 +35,8 @@ Table_Id;
 
 extern Sym_Table syms[NUM_TABLES];
 
-extern void sym_id_add PARAMS ((const char *, Table_Id));
-extern void sym_id_parse PARAMS ((void));
-extern bfd_boolean sym_id_arc_is_present PARAMS ((Sym_Table *, Sym *, Sym *));
+extern void sym_id_add (const char *, Table_Id);
+extern void sym_id_parse (void);
+extern bfd_boolean sym_id_arc_is_present (Sym_Table *, Sym *, Sym *);
 
 #endif /* sym_ids_h */
