@@ -2,12 +2,13 @@ SCRIPT_NAME=elf
 ELFSIZE=64
 TEMPLATE_NAME=elf32
 OUTPUT_FORMAT="elf64-sparc"
-MAXPAGESIZE=0x100000
-COMMONPAGESIZE=0x2000
+MAXPAGESIZE="CONSTANT (MAXPAGESIZE)"
+COMMONPAGESIZE="CONSTANT (COMMONPAGESIZE)"
 ARCH="sparc:v9"
 MACHINE=
 DATA_PLT=
 GENERATE_SHLIB_SCRIPT=yes
+GENERATE_PIE_SCRIPT=yes
 NOP=0x01000000
 NO_SMALL_DATA=yes
 
@@ -42,9 +43,9 @@ case "$EMULATION_NAME" in
   *64*)
     case "$target" in
       sparc*-linux*)
-	suffix=64 ;;
+	LIBPATH_SUFFIX=64 ;;
       sparc*-solaris*)
-	suffix=/sparcv9 ;;
+	LIBPATH_SUFFIX=/sparcv9 ;;
     esac
     ;;
 esac
