@@ -1,6 +1,6 @@
 /* ld.h -- general linker header file
    Copyright 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000,
-   2001, 2002, 2003, 2004, 2005
+   2001, 2002, 2003, 2004, 2005, 2006, 2007
    Free Software Foundation, Inc.
 
    This file is part of GLD, the Gnu Linker.
@@ -81,8 +81,6 @@ void *alloca ();
 # define N_(String) (String)
 #endif
 
-#include "bin-bugs.h"
-
 /* Look in this environment name for the linker to pretend to be */
 #define EMULATION_ENVIRON "LDEMULATION"
 /* If in there look for the strings: */
@@ -149,22 +147,6 @@ typedef struct {
   bfd_boolean inhibit_common_definition;
   bfd_boolean relax;
 
-  /* Name of runtime interpreter to invoke.  */
-  char *interpreter;
-
-  /* Name to give runtime libary from the -soname argument.  */
-  char *soname;
-
-  /* Runtime library search path from the -rpath argument.  */
-  char *rpath;
-
-  /* Link time runtime library search path from the -rpath-link
-     argument.  */
-  char *rpath_link;
-
-  /* Big or little endian as set on command line.  */
-  enum { ENDIAN_UNSET = 0, ENDIAN_BIG, ENDIAN_LITTLE } endian;
-
   /* If TRUE, build MIPS embedded PIC relocation tables in the output
      file.  */
   bfd_boolean embedded_relocs;
@@ -179,6 +161,54 @@ typedef struct {
      files.  */
   bfd_boolean warn_mismatch;
 
+  /* Warn on attempting to open an incompatible library during a library
+     search.  */
+  bfd_boolean warn_search_mismatch;
+
+
+  /* If TRUE (the default) check section addresses, once compute,
+     fpor overlaps.  */
+  bfd_boolean check_section_addresses;
+
+  /* If TRUE allow the linking of input files in an unknown architecture
+     assuming that the user knows what they are doing.  This was the old
+     behaviour of the linker.  The new default behaviour is to reject such
+     input files.  */
+  bfd_boolean accept_unknown_input_arch;
+
+  /* Big or little endian as set on command line.  */
+  enum { ENDIAN_UNSET = 0, ENDIAN_BIG, ENDIAN_LITTLE } endian;
+
+  /* -Bsymbolic and -Bsymbolic-functions, as set on command line.  */
+  enum
+    {
+      symbolic_unset = 0,
+      symbolic,
+      symbolic_functions,
+    } symbolic;
+
+  /* --dynamic-list, --dynamic-list-cpp-new, --dynamic-list-cpp-typeinfo
+     and --dynamic-list FILE, as set on command line.  */
+  enum
+    {
+      dynamic_list_unset = 0,
+      dynamic_list_data,
+      dynamic_list
+    } dynamic_list;
+
+  /* Name of runtime interpreter to invoke.  */
+  char *interpreter;
+
+  /* Name to give runtime libary from the -soname argument.  */
+  char *soname;
+
+  /* Runtime library search path from the -rpath argument.  */
+  char *rpath;
+
+  /* Link time runtime library search path from the -rpath-link
+     argument.  */
+  char *rpath_link;
+
   /* Name of shared object whose symbol table should be filtered with
      this shared object.  From the --filter option.  */
   char *filter_shlib;
@@ -191,20 +221,8 @@ typedef struct {
      .exports sections.  */
   char *version_exports_section;
 
-  /* If TRUE (the default) check section addresses, once compute,
-     fpor overlaps.  */
-  bfd_boolean check_section_addresses;
-
-  /* If TRUE allow the linking of input files in an unknown architecture
-     assuming that the user knows what they are doing.  This was the old
-     behaviour of the linker.  The new default behaviour is to reject such
-     input files.  */
-  bfd_boolean accept_unknown_input_arch;
-
-  /* If TRUE reduce memory overheads, at the expense of speed.
-     This will cause map file generation to use an O(N^2) algorithm.  */
-  bfd_boolean reduce_memory_overheads;
-
+  /* Default linker script.  */
+  char *default_script;
 } args_type;
 
 extern args_type command_line;
@@ -212,7 +230,6 @@ extern args_type command_line;
 typedef int token_code_type;
 
 typedef struct {
-  bfd_size_type specified_data_size;
   bfd_boolean magic_demand_paged;
   bfd_boolean make_executable;
 
@@ -252,24 +269,35 @@ typedef struct {
 
   bfd_boolean text_read_only;
 
-  char *map_filename;
-  FILE *map_file;
-
   bfd_boolean stats;
 
   /* If set, orphan input sections will be mapped to separate output
      sections.  */
   bfd_boolean unique_orphan_sections;
 
-  unsigned int split_by_reloc;
-  bfd_size_type split_by_file;
-
   /* If set, only search library directories explicitly selected
      on the command line.  */
   bfd_boolean only_cmd_line_lib_dirs;
 
+  /* The rpath separation character.  Usually ':'.  */
+  char rpath_separator;
+
+  char *map_filename;
+  FILE *map_file;
+
+  unsigned int split_by_reloc;
+  bfd_size_type split_by_file;
+
+  bfd_size_type specified_data_size;
+
   /* The size of the hash table to use.  */
   bfd_size_type hash_table_size;
+
+  /* The maximum page size for ELF.  */
+  bfd_vma maxpagesize;
+
+  /* The common page size for ELF.  */
+  bfd_vma commonpagesize;
 } ld_config_type;
 
 extern ld_config_type config;
@@ -282,6 +310,7 @@ extern int parsing_defsym;
 
 extern int yyparse (void);
 extern void add_cref (const char *, bfd *, asection *, bfd_vma);
+extern bfd_boolean handle_asneeded_cref (bfd *, enum notice_asneeded_action);
 extern void output_cref (FILE *);
 extern void check_nocrossrefs (void);
 extern void ld_abort (const char *, int, const char *) ATTRIBUTE_NORETURN;
