@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU General Public License
    along with GLD; see the file COPYING.  If not, write to
-   the Free Software Foundation, 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   the Free Software Foundation, 51 Franklin Street - Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #ifndef LDFILE_H
 #define LDFILE_H
@@ -54,8 +54,6 @@ extern void ldfile_open_file
   (struct lang_input_statement_struct *);
 extern bfd_boolean ldfile_try_open_bfd
   (const char *, struct lang_input_statement_struct *);
-extern FILE *ldfile_find_command_file
-  (const char *name, const char *extend);
 extern void ldfile_set_output_arch
   (const char *, enum bfd_architecture);
 extern bfd_boolean ldfile_open_file_search
