@@ -1,6 +1,6 @@
 /* ldlang.h - linker command language support
    Copyright 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000,
-   2001, 2002, 2003
+   2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
 
    This file is part of GLD, the Gnu Linker.
@@ -17,13 +17,16 @@
 
    You should have received a copy of the GNU General Public License
    along with GLD; see the file COPYING.  If not, write to the Free
-   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+   Software Foundation, 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
 #ifndef LDLANG_H
 #define LDLANG_H
 
-typedef enum {
+#define DEFAULT_MEMORY_REGION   "*default*"
+
+typedef enum
+{
   lang_input_file_is_l_enum,
   lang_input_file_is_symbols_only_enum,
   lang_input_file_is_marker_enum,
@@ -32,31 +35,36 @@ typedef enum {
   lang_input_file_is_file_enum
 } lang_input_file_enum_type;
 
-struct _fill_type {
+struct _fill_type
+{
   size_t size;
   unsigned char data[1];
 };
 
-typedef struct statement_list {
+typedef struct statement_list
+{
   union lang_statement_union *head;
   union lang_statement_union **tail;
 } lang_statement_list_type;
 
-typedef struct memory_region_struct {
+typedef struct memory_region_struct
+{
   char *name;
   struct memory_region_struct *next;
   bfd_vma origin;
   bfd_size_type length;
   bfd_vma current;
-  bfd_size_type old_length;
+  union lang_statement_union *last_os;
   flagword flags;
   flagword not_flags;
   bfd_boolean had_full_message;
 } lang_memory_region_type;
 
-typedef struct lang_statement_header_struct {
+typedef struct lang_statement_header_struct
+{
   union lang_statement_union *next;
-  enum statement_enum {
+  enum statement_enum
+  {
     lang_output_section_statement_enum,
     lang_assignment_statement_enum,
     lang_input_statement_enum,
@@ -77,62 +85,56 @@ typedef struct lang_statement_header_struct {
   } type;
 } lang_statement_header_type;
 
-typedef struct {
+typedef struct
+{
   lang_statement_header_type header;
   union etree_union *exp;
 } lang_assignment_statement_type;
 
-typedef struct lang_target_statement_struct {
+typedef struct lang_target_statement_struct
+{
   lang_statement_header_type header;
   const char *target;
 } lang_target_statement_type;
 
-typedef struct lang_output_statement_struct {
+typedef struct lang_output_statement_struct
+{
   lang_statement_header_type header;
   const char *name;
 } lang_output_statement_type;
 
 /* Section types specified in a linker script.  */
 
-enum section_type {
+enum section_type
+{
   normal_section,
-  dsect_section,
-  copy_section,
+  overlay_section,
   noload_section,
-  info_section,
-  overlay_section
+  noalloc_section
 };
 
-/* This structure holds a list of program headers describing segments
-   in which this section should be placed.  */
+/* This structure holds a list of program headers describing
+   segments in which this section should be placed.  */
 
-struct lang_output_section_phdr_list {
+typedef struct lang_output_section_phdr_list
+{
   struct lang_output_section_phdr_list *next;
   const char *name;
   bfd_boolean used;
-};
+} lang_output_section_phdr_list;
 
-typedef struct lang_output_section_statement_struct {
+typedef struct lang_output_section_statement_struct
+{
   lang_statement_header_type header;
-  union etree_union *addr_tree;
   lang_statement_list_type children;
-  const char *memspec;
-  union lang_statement_union *next;
+  struct lang_output_section_statement_struct *next;
+  struct lang_output_section_statement_struct *prev;
   const char *name;
-
-  bfd_boolean processed;
-
   asection *bfd_section;
-  flagword flags;		/* Or together of all input sections */
-  enum section_type sectype;
-  struct memory_region_struct *region;
-  struct memory_region_struct *lma_region;
-  size_t block_value;
+  lang_memory_region_type *region;
+  lang_memory_region_type *lma_region;
   fill_type *fill;
-
-  int subsection_alignment;	/* alignment of components */
-  int section_alignment;	/* alignment of start of section */
-
+  union etree_union *addr_tree;
   union etree_union *load_base;
 
   /* If non-null, an expression to evaluate after setting the section's
@@ -141,36 +143,55 @@ typedef struct lang_output_section_statement_struct {
      to move '.' past all the overlaid sections.  */
   union etree_union *update_dot_tree;
 
-  struct lang_output_section_phdr_list *phdrs;
+  lang_output_section_phdr_list *phdrs;
+
+  unsigned int block_value;
+  int subsection_alignment;	/* Alignment of components.  */
+  int section_alignment;	/* Alignment of start of section.  */
+  int constraint;
+  flagword flags;
+  enum section_type sectype;
+  unsigned int processed_vma : 1;
+  unsigned int processed_lma : 1;
+  unsigned int all_input_readonly : 1;
+  /* If this section should be ignored.  */
+  unsigned int ignored : 1; 
+  /* If there is a symbol relative to this section.  */
+  unsigned int section_relative_symbol : 1; 
 } lang_output_section_statement_type;
 
-typedef struct {
+typedef struct
+{
   lang_statement_header_type header;
 } lang_common_statement_type;
 
-typedef struct {
+typedef struct
+{
   lang_statement_header_type header;
 } lang_object_symbols_statement_type;
 
-typedef struct {
+typedef struct
+{
   lang_statement_header_type header;
   fill_type *fill;
   int size;
   asection *output_section;
 } lang_fill_statement_type;
 
-typedef struct {
+typedef struct
+{
   lang_statement_header_type header;
   unsigned int type;
   union etree_union *exp;
   bfd_vma value;
   asection *output_section;
-  bfd_vma output_vma;
+  bfd_vma output_offset;
 } lang_data_statement_type;
 
 /* Generate a reloc in the output file.  */
 
-typedef struct {
+typedef struct
+{
   lang_statement_header_type header;
 
   /* Reloc to generate.  */
@@ -179,12 +200,12 @@ typedef struct {
   /* Reloc howto structure.  */
   reloc_howto_type *howto;
 
-  /* Section to generate reloc against.  Exactly one of section and
-     name must be NULL.  */
+  /* Section to generate reloc against.
+     Exactly one of section and name must be NULL.  */
   asection *section;
 
-  /* Name of symbol to generate reloc against.  Exactly one of section
-     and name must be NULL.  */
+  /* Name of symbol to generate reloc against.
+     Exactly one of section and name must be NULL.  */
   const char *name;
 
   /* Expression for addend.  */
@@ -196,22 +217,22 @@ typedef struct {
   /* Output section where reloc should be performed.  */
   asection *output_section;
 
-  /* VMA within output section.  */
-  bfd_vma output_vma;
+  /* Offset within output section.  */
+  bfd_vma output_offset;
 } lang_reloc_statement_type;
 
-typedef struct lang_input_statement_struct {
+typedef struct lang_input_statement_struct
+{
   lang_statement_header_type header;
   /* Name of this file.  */
   const char *filename;
-  /* Name to use for the symbol giving address of text start */
-  /* Usually the same as filename, but for a file spec'd with -l
-     this is the -l switch itself rather than the filename.  */
+  /* Name to use for the symbol giving address of text start.
+     Usually the same as filename, but for a file spec'd with
+     -l this is the -l switch itself rather than the filename.  */
   const char *local_sym_name;
 
   bfd *the_bfd;
 
-  bfd_boolean closed;
   file_ptr passive_position;
 
   /* Symbol table of the file.  */
@@ -220,72 +241,108 @@ typedef struct lang_input_statement_struct {
 
   /* Point to the next file - whatever it is, wanders up and down
      archives */
-
   union lang_statement_union *next;
-  /* Point to the next file, but skips archive contents */
+
+  /* Point to the next file, but skips archive contents.  */
   union lang_statement_union *next_real_file;
 
-  bfd_boolean is_archive;
+  const char *target;
+
+  unsigned int closed : 1;
+  unsigned int is_archive : 1;
 
   /* 1 means search a set of directories for this file.  */
-  bfd_boolean search_dirs_flag;
+  unsigned int search_dirs_flag : 1;
 
   /* 1 means this was found in a search directory marked as sysrooted,
      if search_dirs_flag is false, otherwise, that it should be
      searched in ld_sysroot before any other location, as long as it
      starts with a slash.  */
-  bfd_boolean sysrooted;
+  unsigned int sysrooted : 1;
 
   /* 1 means this is base file of incremental load.
      Do not load this file's text or data.
      Also default text_start to after this file's bss.  */
-
-  bfd_boolean just_syms_flag;
+  unsigned int just_syms_flag : 1;
 
   /* Whether to search for this entry as a dynamic archive.  */
-  bfd_boolean dynamic;
+  unsigned int dynamic : 1;
+
+  /* Whether DT_NEEDED tags should be added for dynamic libraries in
+     DT_NEEDED tags from this entry.  */
+  unsigned int add_needed : 1;
+
+  /* Whether this entry should cause a DT_NEEDED tag only when
+     satisfying references from regular files, or always.  */
+  unsigned int as_needed : 1;
 
   /* Whether to include the entire contents of an archive.  */
-  bfd_boolean whole_archive;
+  unsigned int whole_archive : 1;
 
-  bfd_boolean loaded;
+  unsigned int loaded : 1;
 
-#if 0
-  unsigned int globals_in_this_file;
-#endif
-  const char *target;
-  bfd_boolean real;
+  unsigned int real : 1;
 } lang_input_statement_type;
 
-typedef struct {
+typedef struct
+{
   lang_statement_header_type header;
   asection *section;
-  lang_input_statement_type *ifile;
-
 } lang_input_section_type;
 
-typedef struct {
+typedef struct
+{
   lang_statement_header_type header;
   asection *section;
   union lang_statement_union *file;
 } lang_afile_asection_pair_statement_type;
 
-typedef struct lang_wild_statement_struct {
+typedef struct lang_wild_statement_struct lang_wild_statement_type;
+
+typedef void (*callback_t) (lang_wild_statement_type *, struct wildcard_list *,
+			    asection *, lang_input_statement_type *, void *);
+
+typedef void (*walk_wild_section_handler_t) (lang_wild_statement_type *,
+					     lang_input_statement_type *,
+					     callback_t callback,
+					     void *data);
+
+typedef bfd_boolean (*lang_match_sec_type_func) (bfd *, const asection *,
+						 bfd *, const asection *);
+
+/* Binary search tree structure to efficiently sort sections by
+   name.  */
+typedef struct lang_section_bst
+{
+  asection *section;
+  struct lang_section_bst *left;
+  struct lang_section_bst *right;
+} lang_section_bst_type;
+
+struct lang_wild_statement_struct
+{
   lang_statement_header_type header;
   const char *filename;
   bfd_boolean filenames_sorted;
   struct wildcard_list *section_list;
   bfd_boolean keep_sections;
   lang_statement_list_type children;
-} lang_wild_statement_type;
 
-typedef struct lang_address_statement_struct {
+  walk_wild_section_handler_t walk_wild_section_handler;
+  struct wildcard_list *handler_data[4];
+  lang_section_bst_type *tree;
+};
+
+typedef struct lang_address_statement_struct
+{
   lang_statement_header_type header;
   const char *section_name;
   union etree_union *address;
+  const segment_type *segment;
 } lang_address_statement_type;
 
-typedef struct {
+typedef struct
+{
   lang_statement_header_type header;
   bfd_vma output_offset;
   size_t size;
@@ -298,12 +355,14 @@ typedef struct {
    symbols are found.  The effect is to search a group of libraries as
    though they were a single library.  */
 
-typedef struct {
+typedef struct
+{
   lang_statement_header_type header;
   lang_statement_list_type children;
 } lang_group_statement_type;
 
-typedef union lang_statement_union {
+typedef union lang_statement_union
+{
   lang_statement_header_type header;
   lang_wild_statement_type wild_statement;
   lang_data_statement_type data_statement;
@@ -326,7 +385,8 @@ typedef union lang_statement_union {
 /* This structure holds information about a program header, from the
    PHDRS command in the linker script.  */
 
-struct lang_phdr {
+struct lang_phdr
+{
   struct lang_phdr *next;
   const char *name;
   unsigned long type;
@@ -336,19 +396,23 @@ struct lang_phdr {
   etree_type *flags;
 };
 
+extern struct lang_phdr *lang_phdr_list;
+
 /* This structure is used to hold a list of sections which may not
    cross reference each other.  */
 
-struct lang_nocrossref {
+typedef struct lang_nocrossref
+{
   struct lang_nocrossref *next;
   const char *name;
-};
+} lang_nocrossref_type;
 
 /* The list of nocrossref lists.  */
 
-struct lang_nocrossrefs {
+struct lang_nocrossrefs
+{
   struct lang_nocrossrefs *next;
-  struct lang_nocrossref *list;
+  lang_nocrossref_type *list;
 };
 
 extern struct lang_nocrossrefs *nocrossref_list;
@@ -356,12 +420,31 @@ extern struct lang_nocrossrefs *nocrossref_list;
 /* This structure is used to hold a list of input section names which
    will not match an output section in the linker script.  */
 
-struct unique_sections {
+struct unique_sections
+{
   struct unique_sections *next;
   const char *name;
 };
 
-extern struct unique_sections *unique_section_list;
+/* This structure records symbols for which we need to keep track of
+   definedness for use in the DEFINED () test.  */
+
+struct lang_definedness_hash_entry
+{
+  struct bfd_hash_entry root;
+  int iteration;
+};
+
+/* Used by place_orphan to keep track of orphan sections and statements.  */
+
+struct orphan_save {
+  const char *name;
+  flagword flags;
+  lang_output_section_statement_type *os;
+  asection **section;
+  lang_statement_union_type **stmt;
+  lang_output_section_statement_type **os_tail;
+};
 
 extern lang_output_section_statement_type *abs_output_section;
 extern lang_statement_list_type lang_output_section_statement;
@@ -374,72 +457,75 @@ extern struct bfd_sym_chain entry_symbol;
 extern const char *entry_section;
 extern bfd_boolean entry_from_cmdline;
 extern lang_statement_list_type file_chain;
+extern lang_statement_list_type input_file_chain;
+
+extern int lang_statement_iteration;
 
 extern void lang_init
-  PARAMS ((void));
-extern struct memory_region_struct *lang_memory_region_lookup
-  PARAMS ((const char *const));
-extern struct memory_region_struct *lang_memory_region_default
-  PARAMS ((asection *));
+  (void);
+extern void lang_finish
+  (void);
+extern lang_memory_region_type *lang_memory_region_lookup
+  (const char *const, bfd_boolean);
+extern lang_memory_region_type *lang_memory_region_default
+  (asection *);
 extern void lang_map
-  PARAMS ((void));
+  (void);
 extern void lang_set_flags
-  PARAMS ((lang_memory_region_type *, const char *, int));
+  (lang_memory_region_type *, const char *, int);
 extern void lang_add_output
-  PARAMS ((const char *, int from_script));
+  (const char *, int from_script);
 extern lang_output_section_statement_type *lang_enter_output_section_statement
-  PARAMS ((const char *output_section_statement_name,
-	   etree_type *address_exp,
-	   enum section_type sectype,
-	   bfd_vma block_value,
-	   etree_type *align,
-	   etree_type *subalign,
-	   etree_type *));
+  (const char *output_section_statement_name,
+   etree_type *address_exp,
+   enum section_type sectype,
+   etree_type *align,
+   etree_type *subalign,
+   etree_type *, int);
 extern void lang_final
-  PARAMS ((void));
+  (void);
 extern void lang_process
-  PARAMS ((void));
+  (void);
 extern void lang_section_start
-  PARAMS ((const char *, union etree_union *));
+  (const char *, union etree_union *, const segment_type *);
 extern void lang_add_entry
-  PARAMS ((const char *, bfd_boolean));
+  (const char *, bfd_boolean);
+extern void lang_default_entry
+  (const char *);
 extern void lang_add_target
-  PARAMS ((const char *));
+  (const char *);
 extern void lang_add_wild
-  PARAMS ((struct wildcard_spec *, struct wildcard_list *, bfd_boolean));
+  (struct wildcard_spec *, struct wildcard_list *, bfd_boolean);
 extern void lang_add_map
-  PARAMS ((const char *));
+  (const char *);
 extern void lang_add_fill
-  PARAMS ((fill_type *));
-extern lang_assignment_statement_type * lang_add_assignment
-  PARAMS ((union etree_union *));
+  (fill_type *);
+extern lang_assignment_statement_type *lang_add_assignment
+  (union etree_union *);
 extern void lang_add_attribute
-  PARAMS ((enum statement_enum));
+  (enum statement_enum);
 extern void lang_startup
-  PARAMS ((const char *));
+  (const char *);
 extern void lang_float
-  PARAMS ((bfd_boolean));
+  (bfd_boolean);
 extern void lang_leave_output_section_statement
-  PARAMS ((fill_type *, const char *, struct lang_output_section_phdr_list *,
-           const char *));
+  (fill_type *, const char *, lang_output_section_phdr_list *,
+   const char *);
 extern void lang_abs_symbol_at_end_of
-  PARAMS ((const char *, const char *));
+  (const char *, const char *);
 extern void lang_abs_symbol_at_beginning_of
-  PARAMS ((const char *, const char *));
+  (const char *, const char *);
 extern void lang_statement_append
-  PARAMS ((struct statement_list *, union lang_statement_union *,
-	   union lang_statement_union **));
+  (lang_statement_list_type *, lang_statement_union_type *,
+   lang_statement_union_type **);
 extern void lang_for_each_input_file
-  PARAMS ((void (*dothis) (lang_input_statement_type *)));
+  (void (*dothis) (lang_input_statement_type *));
 extern void lang_for_each_file
-  PARAMS ((void (*dothis) (lang_input_statement_type *)));
+  (void (*dothis) (lang_input_statement_type *));
 extern void lang_reset_memory_regions
-  PARAMS ((void));
-extern bfd_vma lang_do_assignments
-  PARAMS ((lang_statement_union_type * s,
-	   lang_output_section_statement_type *output_section_statement,
-	   fill_type *fill,
-	   bfd_vma dot));
+  (void);
+extern void lang_do_assignments
+  (void);
 
 #define LANG_FOR_EACH_INPUT_STATEMENT(statement)			\
   lang_input_statement_type *statement;					\
@@ -448,80 +534,99 @@ extern bfd_vma lang_do_assignments
        statement = (lang_input_statement_type *) statement->next)	\
 
 extern void lang_process
-  PARAMS ((void));
+  (void);
 extern void ldlang_add_file
-  PARAMS ((lang_input_statement_type *));
+  (lang_input_statement_type *);
 extern lang_output_section_statement_type *lang_output_section_find
-  PARAMS ((const char * const));
+  (const char * const);
+extern lang_output_section_statement_type *lang_output_section_find_by_flags
+  (const asection *, lang_output_section_statement_type **,
+   lang_match_sec_type_func);
+extern lang_output_section_statement_type *lang_insert_orphan
+  (asection *, const char *, lang_output_section_statement_type *,
+   struct orphan_save *, etree_type *, lang_statement_list_type *);
 extern lang_input_statement_type *lang_add_input_file
-  PARAMS ((const char *name, lang_input_file_enum_type file_type,
-	   const char *target));
+  (const char *, lang_input_file_enum_type, const char *);
 extern void lang_add_keepsyms_file
-  PARAMS ((const char *filename));
+  (const char *);
 extern lang_output_section_statement_type *
   lang_output_section_statement_lookup
-  PARAMS ((const char * const name));
+  (const char *const);
 extern void ldlang_add_undef
-  PARAMS ((const char *const name));
+  (const char *const);
 extern void lang_add_output_format
-  PARAMS ((const char *, const char *, const char *, int from_script));
+  (const char *, const char *, const char *, int);
 extern void lang_list_init
-  PARAMS ((lang_statement_list_type*));
+  (lang_statement_list_type *);
 extern void lang_add_data
-  PARAMS ((int type, union etree_union *));
+  (int type, union etree_union *);
 extern void lang_add_reloc
-  PARAMS ((bfd_reloc_code_real_type reloc, reloc_howto_type *howto,
-	   asection *section, const char *name, union etree_union *addend));
+  (bfd_reloc_code_real_type, reloc_howto_type *, asection *, const char *,
+   union etree_union *);
 extern void lang_for_each_statement
-  PARAMS ((void (*func) (lang_statement_union_type *)));
-extern PTR stat_alloc
-  PARAMS ((size_t size));
+  (void (*) (lang_statement_union_type *));
+extern void *stat_alloc
+  (size_t);
+extern void strip_excluded_output_sections
+  (void);
 extern void dprint_statement
-  PARAMS ((lang_statement_union_type *, int));
-extern bfd_vma lang_size_sections
-  PARAMS ((lang_statement_union_type *s,
-	   lang_output_section_statement_type *output_section_statement,
-	   lang_statement_union_type **prev, fill_type *fill,
-	   bfd_vma dot, bfd_boolean *relax, bfd_boolean check_regions));
+  (lang_statement_union_type *, int);
+extern void lang_size_sections
+  (bfd_boolean *, bfd_boolean);
+extern void one_lang_size_sections_pass
+  (bfd_boolean *, bfd_boolean);
 extern void lang_enter_group
-  PARAMS ((void));
+  (void);
 extern void lang_leave_group
-  PARAMS ((void));
+  (void);
 extern void lang_add_section
-  PARAMS ((lang_statement_list_type *ptr, asection *section,
-	   lang_output_section_statement_type *output,
-	   lang_input_statement_type *file));
+  (lang_statement_list_type *, asection *,
+   lang_output_section_statement_type *);
 extern void lang_new_phdr
-  PARAMS ((const char *, etree_type *, bfd_boolean, bfd_boolean, etree_type *,
-	   etree_type *));
+  (const char *, etree_type *, bfd_boolean, bfd_boolean, etree_type *,
+   etree_type *);
 extern void lang_add_nocrossref
-  PARAMS ((struct lang_nocrossref *));
+  (lang_nocrossref_type *);
 extern void lang_enter_overlay
-  PARAMS ((etree_type *));
+  (etree_type *, etree_type *);
 extern void lang_enter_overlay_section
-  PARAMS ((const char *));
+  (const char *);
 extern void lang_leave_overlay_section
-  PARAMS ((fill_type *, struct lang_output_section_phdr_list *));
+  (fill_type *, lang_output_section_phdr_list *);
 extern void lang_leave_overlay
-  PARAMS ((etree_type *, int, fill_type *, const char *,
-	   struct lang_output_section_phdr_list *, const char *));
+  (etree_type *, int, fill_type *, const char *,
+   lang_output_section_phdr_list *, const char *);
 
 extern struct bfd_elf_version_tree *lang_elf_version_info;
 
 extern struct bfd_elf_version_expr *lang_new_vers_pattern
-  PARAMS ((struct bfd_elf_version_expr *, const char *, const char *));
+  (struct bfd_elf_version_expr *, const char *, const char *, bfd_boolean);
 extern struct bfd_elf_version_tree *lang_new_vers_node
-  PARAMS ((struct bfd_elf_version_expr *, struct bfd_elf_version_expr *));
+  (struct bfd_elf_version_expr *, struct bfd_elf_version_expr *);
 extern struct bfd_elf_version_deps *lang_add_vers_depend
-  PARAMS ((struct bfd_elf_version_deps *, const char *));
+  (struct bfd_elf_version_deps *, const char *);
 extern void lang_register_vers_node
-  PARAMS ((const char *, struct bfd_elf_version_tree *,
-	   struct bfd_elf_version_deps *));
+  (const char *, struct bfd_elf_version_tree *, struct bfd_elf_version_deps *);
+extern void lang_append_dynamic_list (struct bfd_elf_version_expr *);
+extern void lang_append_dynamic_list_cpp_typeinfo (void);
+extern void lang_append_dynamic_list_cpp_new (void);
 bfd_boolean unique_section_p
-  PARAMS ((const char *));
+  (const asection *);
 extern void lang_add_unique
-  PARAMS ((const char *));
+  (const char *);
 extern const char *lang_get_output_target
-  PARAMS ((void));
+  (void);
+extern void lang_track_definedness (const char *);
+extern int lang_symbol_definition_iteration (const char *);
+extern void lang_update_definedness
+  (const char *, struct bfd_link_hash_entry *);
+
+extern void add_excluded_libs (const char *);
+extern bfd_boolean load_symbols
+  (lang_input_statement_type *, lang_statement_list_type *);
+
+extern bfd_boolean
+ldlang_override_segment_assignment
+  (struct bfd_link_info *, bfd *, asection *, asection *, bfd_boolean);
 
 #endif
