@@ -3,22 +3,22 @@
    Free Software Foundation, Inc.
    Written by DJ Delorie <dj@cygnus.com>
 
-   This file is part of the GNU Binutils.
+   This file is part of GLD, the Gnu Linker.
 
-   This program is free software; you can redistribute it and/or modify
+   GLD is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GLD is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston,
-   MA 02110-1301, USA.  */
+   along with GLD; see the file COPYING.  If not, write to the Free
+   Software Foundation, 51 Franklin Street - Fifth Floor, Boston, MA
+   02110-1301, USA.  */
 
 #include "sysdep.h"
 #include "bfd.h"
@@ -2416,7 +2416,7 @@ pe_dll_generate_implib (def_file *def, const char *impfilename)
       def->exports[i].internal_name = def->exports[i].name;
       n = make_one (def->exports + i, outarch,
 		    ! (def->exports + i)->flag_data);
-      n->archive_next = head;
+      n->next = head;
       head = n;
       def->exports[i].internal_name = internal;
     }
@@ -2427,8 +2427,8 @@ pe_dll_generate_implib (def_file *def, const char *impfilename)
     return;
 
   /* Now stick them all into the archive.  */
-  ar_head->archive_next = head;
-  ar_tail->archive_next = ar_head;
+  ar_head->next = head;
+  ar_tail->next = ar_head;
   head = ar_tail;
 
   if (! bfd_set_archive_head (outarch, head))
@@ -2439,7 +2439,7 @@ pe_dll_generate_implib (def_file *def, const char *impfilename)
 
   while (head != NULL)
     {
-      bfd *n = head->archive_next;
+      bfd *n = head->next;
       bfd_close (head);
       head = n;
     }
