@@ -1,5 +1,6 @@
 /* m68hc11-dis.c -- Motorola 68HC11 & 68HC12 disassembly
-   Copyright 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright 1999, 2000, 2001, 2002, 2003, 2006
+   Free Software Foundation, Inc.
    Written by Stephane Carrez (stcarrez@nerim.fr)
 
 This program is free software; you can redistribute it and/or modify
@@ -692,11 +693,11 @@ print_insn (bfd_vma memaddr, struct disassemble_info* info, int arch)
          opcode table content.  */
       if (format & ~(M6811_OP_PAGE4 | M6811_OP_PAGE3 | M6811_OP_PAGE2))
 	{
-	  (*info->fprintf_func) (info->stream, "; Error, format: %x", format);
+	  (*info->fprintf_func) (info->stream, "; Error, format: %lx", format);
 	}
       if (pos != opcode->size)
 	{
-	  (*info->fprintf_func) (info->stream, "; Error, size: %d expect %d",
+	  (*info->fprintf_func) (info->stream, "; Error, size: %ld expect %d",
 				 pos, opcode->size);
 	}
 #endif
